/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.tracker;

import java.util.LinkedList;
import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.FrameworkUtil;
import org.osgi.util.tracker.BundleTracker;

public class BundleTrackerUtils {

	public static class BundleTrackerBuilder<T> {
	
		protected UniformBundleTrackerCustomizer<T> customizer; 
		
		protected List<BundleFilter<?>> filters = new LinkedList<>();
		
		protected BundleContext bundleContext = FrameworkUtil.getBundle(BundleTrackerUtils.class).getBundleContext();
		
		protected int bundleState = Bundle.ACTIVE;
		
		public BundleTrackerBuilder<T> customizer(UniformBundleTrackerCustomizer<T> customizer){
			this.customizer = customizer;
			return this;
		} 
		
		public BundleTrackerBuilder<T> bundleContext(BundleContext bundleContext){
			this.bundleContext = bundleContext;
			return this;
		} 
		
		public BundleTrackerBuilder<T> addFilter(BundleFilter<?> bundleFilter){
			this.filters.add(bundleFilter);
			return this;
		}
		
		public BundleTrackerBuilder<T> bundleState(int state){
			this.bundleState = state;
			return this;
		} 
		
		public BundleTracker<T> get(){
			if (customizer == null)
				return null;
			if (customizer instanceof BundleContextAware)
				((BundleContextAware)customizer).setBundleContext(bundleContext);
			for (BundleFilter<?> filter: this.filters) {
				if (filter instanceof BundleContextAware)
					((BundleContextAware)filter).setBundleContext(bundleContext);
				customizer.addFilter(filter);
			}
			return new BundleTracker<T>(bundleContext, bundleState, customizer);
		}
	}
	
	public static <T> BundleTrackerBuilder<T> createBuilder(){
		return new BundleTrackerBuilder<T>(); 
	} 
	
}
