/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.tracker.filter;

import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.wiring.BundleCapability;
import org.osgi.framework.wiring.BundleRevision;

public class BundleCapabilityFilter extends AbstractBundleFilter<BundleCapability, BundleCapabilityFilter> {

	public static final String DEFAULT_FILTER_ID = "bundle.capabilities";
	
	protected String namespace;
	
	@Override
	protected String getDefaultFilterId() {
		return DEFAULT_FILTER_ID;
	}
	
	@Override
	public List<BundleCapability> checkBundle(Bundle bundle, BundleEvent event) {
		BundleRevision revision = bundle.adapt(BundleRevision.class);
		if (revision == null)
			return null;
		List<BundleCapability> capabilities = revision.getDeclaredCapabilities(namespace);
		if ((capabilities==null) || (capabilities.size()==0))
			return null;
		return capabilities;
	}

	public void setNamespace(String namespace) {
		this.namespace = namespace;
	}

	public BundleCapabilityFilter namespace(String namespace) {
		setNamespace(namespace);
		return this;
	}
	
}
