/*-
 * ~~~~~~licensing~~~~~~
 * base-support
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.tracker.filter;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;

import ru.entaxy.platform.base.support.CommonUtils;

public class BundleHeaderFilter extends AbstractBundleFilter<String, BundleHeaderFilter> {

	public static final String DEFAULT_FILTER_ID = "bundle.header";
	
	protected String header;
	
	protected String value = null;
	
	@Override
	protected String getDefaultFilterId() {
		return DEFAULT_FILTER_ID;
	}

	@Override
	public List<String> checkBundle(Bundle bundle, BundleEvent event) {
		Map<String, String> headers = CommonUtils.getDictionaryAsMap(bundle.getHeaders());
		if (!headers.containsKey(header))
			return null;
		String headerValue = headers.get(header);
		if (value != null)
			if (!value.equals(headerValue))
				return null;
		return Arrays.asList(new String[] {headerValue});
	}
	
	public String getHeader() {
		return header;
	}

	public void setHeader(String header) {
		this.header = header;
	}

	public BundleHeaderFilter header(String headerValue) {
		this.setHeader(headerValue);
		return this;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	};
	
	public BundleHeaderFilter value(String valueValue) {
		this.setValue(valueValue);
		return this;
	};
}
