/*-
 * ~~~~~~licensing~~~~~~
 * adapters-core
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.base.connecting.adapter.core.metadata;

import java.util.HashMap;
import java.util.Map;

import com.google.gson.JsonObject;

import ru.entaxy.platform.base.support.JSONUtils;

public class AdapterGeneratorElement {
    protected String type;
    
    // Default generator is defined in GeneratorFactory
    // no need to redefine it here
    protected String generator = "";

    protected JsonObject config;
    protected Map<String, Object> configMap = new HashMap<>();
    
    // we can postpone generator linking until it's really needed
    protected boolean lazy = false;
    
    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getGenerator() {
        return generator;
    }

    public void setGenerator(String generator) {
        this.generator = generator;
    }

    public Map<String, Object> getConfigMap() {
    	if (configMap.isEmpty() && (this.config != null))
    		configMap = JSONUtils.element2map(this.config);
		return configMap;
	}

	public void setConfigMap(Map<String, Object> configMap) {
		this.configMap = configMap;
	}

	// actually the method is not called by Gson
	public void setConfig(JsonObject configElement) {
		this.config = configElement; 
		this.configMap = JSONUtils.element2map(configElement);
	}

	public boolean isLazy() {
		return lazy;
	}

	public void setLazy(boolean lazy) {
		this.lazy = lazy;
	}

	public String toString() {
        return "AdapterGeneratorElement: " +
                "{'type': '" + type + "', " +
                "'generator': '" + generator + "'}";
    }
}
