/*-
 * ~~~~~~licensing~~~~~~
 * adapters-core
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.base.connecting.adapter.core.tracker;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.framework.wiring.BundleWiring;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.platform.runtime.base.connecting.adapter.core.api.Adapter;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

public class AdapterHelper {
	
	protected static final Logger log = LoggerFactory.getLogger(AdapterHelper.class);
	
	protected BundleContext bundleContext;
	
	protected List<Object> objects = new ArrayList<>();
	protected List<ServiceRegistration> registrations = new ArrayList<>();
	
	public AdapterHelper(BundleContext bundleContext) {
		this.bundleContext = bundleContext;
	}
	
	public void create(Bundle targetBundle, String className) {
		log.info("Creating adapter in " + targetBundle.getSymbolicName()
			+ " of class " + targetBundle.getClass().getName()
			+ " having context of " + targetBundle.getBundleContext().getClass().getName());
		BundleWiring wiring = targetBundle.adapt(BundleWiring.class);
		ClassLoader cl = wiring.getClassLoader();
		try {
			Class<?> clazz = cl.loadClass(className);
			Constructor<?> constructor = clazz.getConstructor(BundleContext.class);
			Object obj = constructor.newInstance(targetBundle.getBundleContext());
			log.debug("Created object of class {}", obj.getClass().getName());

			Adapter adapter = (Adapter)obj;
			if (!adapter.isInited()) {
				log.warn("Adapter initialization failed.");
				return;
			}
			
			Hashtable<String, String> properties = new Hashtable<>();
			properties.put("adapter.id", adapter.getId());
			properties.put("adapter.name", adapter.getName());
			properties.put("adapter.description", adapter.getDescription());
			
			ServiceRegistration sr = targetBundle.getBundleContext().registerService(new String[] {Adapter.class.getName()}, obj, properties);

//			// print out adapter info
//			String props = "\n";
//			for (Map.Entry<String, String> entry: adapter.getProperties().entrySet())
//				props += entry.getKey() + ": " + entry.getValue() + "\n";
//			String opts = "\n";
//			for (Map.Entry<String, String> entry: adapter.getOptions().entrySet())
//				opts += entry.getKey() + ": " + entry.getValue() + "\n";
//
//			log.info("\n\n\tFINAL ADAPTER INFO: \nID: {}\nDescription: {}\nProperties:{}Options:{}"
//					, adapter.getId()
//					, adapter.getDescription()
//					, props
//					, opts);
			
		} catch (Exception e) {
			log.error("Can't create adapter:\n", e);
		}
	}
}
