/*-
 * ~~~~~~licensing~~~~~~
 * generator-api
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.base.generator.template.impl;

import java.net.URL;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;

import ru.entaxy.base.generator.template.TemplateService;
import ru.entaxy.esb.system.core.template.Template;

@Component(immediate = true)
public class LegacyTemplateAdapter {

	BundleContext bundleContext = null;
	
	List<Template> delayedTemplates = new ArrayList<>();
	
	Map<Template, ServiceRegistration<ru.entaxy.base.generator.template.Template>> services = new HashMap<>();
	
	protected class TemplateWrapper implements ru.entaxy.base.generator.template.Template {
		
		Template template;
		
		public TemplateWrapper(Template template) {
			this.template = template;
		}

		@Override
		public URL getTemplateLocation() {
			return template.getTemplateLocation();
		}

		@Override
		public String getId() {
			return template.getTemplateName();
		}

		@Override
		public String getType() {
			return "ftl";
		}

		@Override
		public String getTemplateName() {
			return template.getTemplateName();
		}

		@Override
		public String getTemplateFileName() {
			return template.getTemplateFileName();
		}

		@Override
		public String getTemplateFullName() {
			return template.getTemplateFileName() + ".ftl";
		}
		
	}
	
	@Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, unbind = "removeTemplate")
	public void addTemplate(Template template) {
		if (bundleContext == null)
			delayedTemplates.add(template);
		else
			wrapTemplate(template);
	}
	
	public void removeTemplate(Template template) {
		this.services.remove(template);
	}
	
	@Activate
	public void activate(BundleContext context) {
		this.bundleContext = context;
		processDelayed();
	}
	
	protected void processDelayed() {
		for (Template t: delayedTemplates)
			wrapTemplate(t);
		delayedTemplates.clear();
	}
	
	protected void wrapTemplate(Template template) {
		URL url = template.getTemplateLocation();
		Dictionary<String, String> props = new Hashtable<>();
		props.put(TemplateService.PROP_ID, template.getTemplateName());
		props.put(TemplateService.PROP_NAME, template.getTemplateName());
		props.put(TemplateService.PROP_DESCRIPTION, template.getTemplateName());
		props.put(TemplateService.PROP_TYPE, "ftl");
		props.put("origin.location", url.toString());
		props.put("origin.filename", template.getTemplateFileName());
		ServiceRegistration<ru.entaxy.base.generator.template.Template> service = 
				this.bundleContext.registerService(ru.entaxy.base.generator.template.Template.class
					, new TemplateWrapper(template)
					, props);
		this.services.put(template, service);
	}
}
