/*-
 * ~~~~~~licensing~~~~~~
 * generator-factory
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.base.connecting.generator.factory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.base.generator.template.Template;
import ru.entaxy.base.generator.template.TemplateAwareGenerator;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generator;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.GeneratorService;
import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.base.support.osgi.OSGIUtils;

public class GeneratorFactory {
    private final static Logger log = LoggerFactory.getLogger(GeneratorFactory.class);
    public static final String DEFAULT_GENERATOR = "ftl";

    public static final Map<String, GeneratorService> generatorServices = new HashMap<>();
    
    public static final Map<String, String> templateAwareGeneratorServices = new HashMap<>();
    
    public static Generator createGenerator(String usageType, BundleContext bundleContext) {
        return createGenerator(DEFAULT_GENERATOR, usageType, bundleContext);
    }

    public static void register(String name, GeneratorService generatorService) {
    	GeneratorFactory.generatorServices.put(name, generatorService);
    	if (generatorService instanceof TemplateAwareGenerator) {
    		TemplateAwareGenerator tag = (TemplateAwareGenerator)generatorService;
    		for (String type: tag.getSupportedTemplateTypes())
    			templateAwareGeneratorServices.put(type, name);
    	}
    }
    
    public static void unregister(String name) {
    	
    	GeneratorFactory.generatorServices.remove(name);
    	List<String> types = templateAwareGeneratorServices.entrySet().stream()
    		.filter(e -> e.getValue().equals(name))
    		.map(e -> e.getKey())
    		.collect(Collectors.toList());
    	for (String type: types)
    		templateAwareGeneratorServices.remove(type);
    }
    
    public static Generator createGenerator(String generatorType, String usageType, BundleContext bundleContext) {
        Map<String, Object> properties = new HashMap<>();
        properties.put(Generator.PROP_USAGE_TYPE, usageType);
        properties.put(Generator.PROP_TARGET_BUNDLE_CONTEXT, bundleContext);
        return createGenerator(generatorType, properties);
    }
    
    public static Generator createGenerator(String generatorType, String usageType, BundleContext bundleContext, Map<String, Object> properties) {
        Map<String, Object> props = new HashMap<>();
        props.putAll(properties);
        properties.put(Generator.PROP_USAGE_TYPE, usageType);
        properties.put(Generator.PROP_TARGET_BUNDLE_CONTEXT, bundleContext);
        return createGenerator(generatorType, props);
    }

    public static TemplateAwareGenerator createGenerator(Template template) {
    	return createGenerator(template, new HashMap<>());
    }
    
    public static TemplateAwareGenerator createGenerator(Template template, Map<String, Object> properties) {
    	String type = templateAwareGeneratorServices.get(template.getType());
    	if (CommonUtils.isValid(type))
    		return (TemplateAwareGenerator)createGenerator(type, properties);
    	return null;
    }
    
    public static Generator createGenerator(String generatorType, Map<String, Object> properties) {
        Generator generator = null;
        String typeToSearch = generatorType;

        if (!CommonUtils.isValid(typeToSearch))
        	typeToSearch = DEFAULT_GENERATOR;
        GeneratorService generatorService = GeneratorFactory.generatorServices.get(typeToSearch);

    	if (generatorService != null)
    		generator = generatorService.getGenerator(properties);
        return generator;
    	
    }
}
