/*-
 * ~~~~~~licensing~~~~~~
 * management-core
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.base.management.core.api;

import javax.management.Descriptor;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanOperationInfo;
import javax.management.NotCompliantMBeanException;
import javax.management.StandardMBean;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.utils.MBeanInfoHelper;

public class AnnotatedMBean<I> extends StandardMBean {

	private static final Logger log = LoggerFactory.getLogger(AnnotatedMBean.class);
	
	protected Class<I> interfaceClass;
	
	protected AnnotatedMBean(Class<I> mbeanInterface) throws NotCompliantMBeanException {
		super(mbeanInterface);
		this.interfaceClass = mbeanInterface;
	}

	@Override
	public MBeanInfo getMBeanInfo() {
		log.info(this.interfaceClass==null?">> INTERFACE IS NULL":">> " + this.interfaceClass.getName());
		MBeanExportPolicy policy = MBeanExportPolicy.IGNORE_ANNOTATIONS;
		MBeanAnnotated anno = this.interfaceClass.getAnnotation(MBeanAnnotated.class);
		if (anno != null)
			policy = anno.policy();
		
		MBeanInfo beanInfo = super.getMBeanInfo();
		if (MBeanExportPolicy.IGNORE_ANNOTATIONS.equals(policy))
			return beanInfo; 
		
		String className = beanInfo.getClassName();
		String description = beanInfo.getDescription();
		MBeanAttributeInfo[] attributes = /* beanInfo.getAttributes() */ getMBeanAttributeInfo(policy);
		MBeanConstructorInfo[] constructors = beanInfo.getConstructors();
		MBeanOperationInfo[] operations = /* beanInfo.getOperations() */ getMBeanOperationInfo(policy);
		MBeanNotificationInfo[] notifications = beanInfo.getNotifications();
		Descriptor descriptor = beanInfo.getDescriptor();
		
		return new MBeanInfo(className, description, attributes, constructors, operations, notifications, descriptor); 
	}
	
	protected MBeanAttributeInfo[] getMBeanAttributeInfo(MBeanExportPolicy policy) {
		return MBeanInfoHelper.getMBeanAttributesInfo(this.interfaceClass, policy);
	} 
	
	protected MBeanOperationInfo[] getMBeanOperationInfo(MBeanExportPolicy policy) {
		return MBeanInfoHelper.getMBeanOperationsInfo(this.interfaceClass, policy);
	}
	
}
