/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.installer.builder;

import java.util.List;
import java.util.Map;

public interface InstallationResult {

	public static final String PROP_BUNDLE_STATUS = "BundleStatus";
	
	public enum Result {
		INSTALLED,
		UPDATED,
		NOT_CHANGED,
		STARTED,
		STOPPED,
		UNINSTALLED,
		FAILED
	}

	public Result getResult();
	
	public Map<String, Object> getProperties();
	
	public Object getObject();
	
	public List<InstallationResult> getSubResults();
	
	public Throwable getError();
	
	public String getMessage();
	
	public default boolean isSuccessful() {
		return !Result.FAILED.equals(getResult());
	}
}
