/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.installer.builder.impl;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import ru.entaxy.platform.core.artifact.installer.builder.InstallationResult;

public class InstallationResultImpl implements InstallationResult {

	public static InstallationResultImpl create() {
		return new InstallationResultImpl();
	}
	
	protected Result result = null;
	
	protected final Map<String, Object> properties = new HashMap<>(); 
	
	protected Object object = null;
	
	protected final List<InstallationResult> subResults = new LinkedList<>();
	
	protected Throwable error = null;
	
	protected String message = null;
	
	@Override
	public Result getResult() {
		return this.result;
	}

	@Override
	public Map<String, Object> getProperties() {
		return this.properties;
	}

	@Override
	public Object getObject() {
		return this.object;
	}

	@Override
	public List<InstallationResult> getSubResults() {
		return this.subResults;
	}

	@Override
	public Throwable getError() {
		return this.error;
	}

	@Override
	public String getMessage() {
		return this.message;
	}
	
	public InstallationResultImpl result(Result result) {
		this.result = result;
		return this;
	}
	
	public InstallationResultImpl object(Object object) {
		this.object = object;
		return this;
	}
	
	public InstallationResultImpl property(String key, Object value) {
		this.properties.put(key, value);
		return this;
	}
	
	public InstallationResultImpl error(Throwable error) {
		this.error = error;
		return this;
	}
	
	public InstallationResultImpl subResult(InstallationResult subresult) {
		this.subResults.add(subresult);
		return this;
	}
	
	public InstallationResultImpl message(String message) {
		this.message = message;
		return this;
	}
	
	public InstallationResultImpl notChanged() {
		return this.result(Result.NOT_CHANGED);
	}
	
	public InstallationResultImpl installed() {
		return this.result(Result.INSTALLED);
	}

	public InstallationResultImpl installed(Object object) {
		object(object);
		return this.result(Result.INSTALLED);
	}

	public InstallationResultImpl uninstalled() {
		return this.result(Result.UNINSTALLED);
	}

	public InstallationResultImpl uninstalled(Object object) {
		object(object);
		return this.result(Result.UNINSTALLED);
	}

	public InstallationResultImpl updated() {
		return this.result(Result.UPDATED);
	}

	public InstallationResultImpl updated(Object object) {
		object(object);
		return this.result(Result.UPDATED);
	}
	
	public InstallationResultImpl started() {
		return this.result(Result.STARTED);
	}

	public InstallationResultImpl started(Object object) {
		object(object);
		return this.result(Result.STARTED);
	}
	
	public InstallationResultImpl stopped() {
		return this.result(Result.STOPPED );
	}

	public InstallationResultImpl stopped(Object object) {
		object(object);
		return this.result(Result.STOPPED);
	}
	
	public InstallationResultImpl failed() {
		return this.result(Result.FAILED);
	}

	public InstallationResultImpl failed(String message) {
		message(message);
		return this.result(Result.FAILED);
	}
	
	
}
