/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.installer.builder.impl;

import ru.entaxy.platform.core.artifact.installer.ArtifactInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.ClusterInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.Installers;
import ru.entaxy.platform.core.artifact.installer.builder.LocalInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.impl.cluster.ClusterInstallerWrapperImpl;

public class InstallersImpl implements Installers {

	protected ArtifactInstaller<LocalInstaller> local = null;
	protected ArtifactInstaller<ClusterInstaller> cluster = null;

	protected boolean localOnlyMode = false;
	
	@Override
	public LocalInstaller local() {
		if (local == null)
			// TODO throw exception
			return null;
		return local.createInstaller();
	}

	@Override
	public ClusterInstaller cluster(String... groupName) {
		if (this.localOnlyMode)
			return new ClusterInstallerWrapperImpl(this.local());
		if (cluster == null)
			// TODO throw exception
			return null;
		return cluster.createInstaller().groups(groupName);
	}

	public ArtifactInstaller<LocalInstaller> getLocal() {
		return local;
	}

	public void setLocal(ArtifactInstaller<LocalInstaller> local) {
		this.local = local;
	}

	public ArtifactInstaller<ClusterInstaller> getCluster() {
		return cluster;
	}

	public void setCluster(ArtifactInstaller<ClusterInstaller> cluster) {
		this.cluster = cluster;
	}

	public boolean isLocalOnlyMode() {
		return localOnlyMode;
	}

	public void setLocalOnlyMode(boolean localOnlyMode) {
		this.localOnlyMode = localOnlyMode;
	}
	
}
