/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.installer.builder.impl.cluster;

import ru.entaxy.platform.core.artifact.DeployedArtifact;
import ru.entaxy.platform.core.artifact.installer.builder.ClusterInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.InstallationResult;
import ru.entaxy.platform.core.artifact.installer.builder.LocalInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.typed.TypedInstaller;
import ru.entaxy.platform.core.artifact.legacy.BundleController;

public class ClusterInstallerWrapperImpl implements ClusterInstaller {
	
	protected LocalInstaller delegate;
	
	public ClusterInstallerWrapperImpl(LocalInstaller delegate) {
		this.delegate = delegate;
	}

	@Override
	public ClusterInstaller artifact(DeployedArtifact artifact) {
		this.delegate.artifact(artifact);
		return this;
	}

	@Override
	public ClusterInstaller sourceLocation(String sourceLocation) {
		this.delegate.sourceLocation(sourceLocation);
		return this;
	}

	@Override
	public ClusterInstaller bundleName(String bundleName) {
		this.delegate.bundleName(bundleName);
		return this;
	}

	@Override
	public <T extends TypedInstaller> T typed(Class<T> type) {
		return this.delegate.typed(type);
	}

	@Override
	public BundleController bundleController() {
		return this.delegate.bundleController();
	}

	@Override
	public InstallationResult install() {
		return this.delegate.install();
	}

	@Override
	public InstallationResult uninstall() {
		return this.delegate.uninstall();
	}

	@Override
	public ClusterInstaller groups(String... groups) {
		// NOOP
		return this;
	}
	
}
