/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.repository.impl.remote;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

public class RemoteRepositoryFactory {

	private static Map<String, Class<? extends AbstractRemoteRepository>> repositoryClasses;
	
	static {
		repositoryClasses = new HashMap<>();
		repositoryClasses.put("nexus", NexusRepository.class);
	}
	
	
	public static RemoteRepository create(RemoteRepositoryDescriptor descriptor) 
			throws NoSuchMethodException, SecurityException, InstantiationException, 
				IllegalAccessException, IllegalArgumentException, InvocationTargetException {
		
		Class<? extends AbstractRemoteRepository> clazz = repositoryClasses.get(descriptor.getType());
		Constructor<? extends AbstractRemoteRepository> constructor = clazz.getConstructor();
		AbstractRemoteRepository remoteRepository = constructor.newInstance();
		remoteRepository.init(descriptor);
		
		return remoteRepository;
	};
	
}
