/*-
 * ~~~~~~licensing~~~~~~
 * connection-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.connection.impl;

import javax.management.NotCompliantMBeanException;
import javax.management.StandardMBean;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

import ru.entaxy.esb.platform.core.management.connection.ConnectionMBean;

public class ConnectionMBeanImpl extends StandardMBean implements ConnectionMBean {

	protected ManagedConnection connection;
	
	protected BundleContext bundleContext;
	
	public ConnectionMBeanImpl(BundleContext bundleContext, ManagedConnection connection) throws NotCompliantMBeanException {
		super(ConnectionMBean.class);
		this.bundleContext = bundleContext;
		this.connection = connection;
	}

	@Override
	public String getName() {
		return connection.getName();
	}

	@Override
	public boolean isPlatform() {
		return connection.isPlatform();
	}

	@Override
	public String getStatus() {
		int state = this.bundleContext.getBundle(connection.getBundleId()).getState();
		switch (state) {
		case Bundle.ACTIVE:
			return "Active";
		case Bundle.INSTALLED:
			return "Installed";
		case Bundle.RESOLVED:
			return "Resolved";
		case Bundle.STARTING:
			return "Starting";
		case Bundle.STOPPING:
			return "Stopping";
		default:
			return "Unknown";
		}
	}

	@Override
	public boolean isLocal() {
		return connection.isLocal;
	}

	@Override
	public boolean isShared() {
		return connection.isShared();
	}
	
	@Override
	public boolean isDeployed() {
		return connection.isDeployed;
	}

	@Override
	public String getBundleId() {
		return connection.getBundleId() + "";
	}
	
}
