/*-
 * ~~~~~~licensing~~~~~~
 * connection-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.connection.tracker;

import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.wiring.BundleCapability;
import org.osgi.framework.wiring.BundleRevision;
import org.osgi.util.tracker.BundleTrackerCustomizer;

import ru.entaxy.esb.platform.core.management.connection.ConnectionsMBean;
import ru.entaxy.esb.platform.core.management.connection.impl.ManagedConnection;
import ru.entaxy.esb.platform.core.management.connection.impl.ManagedConnections;
import ru.entaxy.esb.platform.core.management.connection.impl.ManagedConnectionsListener;
import ru.entaxy.platform.base.support.CommonUtils;

public class DeployedConnectionCustomizer implements BundleTrackerCustomizer<ManagedConnections> {

	protected ManagedConnectionsListener listener;
	
	public DeployedConnectionCustomizer(ManagedConnectionsListener listener) {
		this.listener = listener;
	}
	
	@Override
	public ManagedConnections addingBundle(Bundle bundle, BundleEvent event) {
		BundleRevision revision = bundle.adapt(BundleRevision.class);
		if (revision == null)
			return null;
		List<BundleCapability> capabilities = revision.getDeclaredCapabilities(ConnectionsMBean.CAPABILITY_NAMESPACE);
		if ((capabilities==null) || (capabilities.size()==0))
			return null;
		ManagedConnections result = new ManagedConnections();
		for (BundleCapability capability: capabilities) {
			Object val = capability.getAttributes().get("name");
			String name = val==null?"":val.toString();
			if (!CommonUtils.isValid(name))
				continue;
			val = capability.getAttributes().get("platform");
			boolean isPlatform = val==null?false:"true".equals(val.toString());
			ManagedConnection mc = (new ManagedConnection())
					.name(name)
					.platform(isPlatform)
					.deployed(true)
					.local(isPlatform)
					.shared(true)
					.bundleId(bundle.getBundleId());
			result.managedConnections.add(mc);
		}
		listener.addded(result);
		return result;
	}

	@Override
	public void modifiedBundle(Bundle bundle, BundleEvent event, ManagedConnections object) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void removedBundle(Bundle bundle, BundleEvent event, ManagedConnections object) {
		listener.removed(object);
	}

}
