/*-
 * ~~~~~~licensing~~~~~~
 * profile-management-service-adapters
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.adapters;

import java.util.Map;

import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;

import ru.entaxy.esb.system.connector.entity.Connector;
import ru.entaxy.esb.system.connector.impl.ConnectorService;
import ru.entaxy.esb.system.jpa.SystemService;
import ru.entaxy.esb.system.jpa.entity.System;
import ru.entaxy.esb.system.management.bundle.jpa.dto.BundleDto;
import ru.entaxy.esb.system.management.bundle.jpa.dto.BundleStatus;
import ru.entaxy.esb.system.management.connector.manager.ConnectorManager;
import ru.entaxy.esb.system.management.connector.manager.dto.ConnectorDto;

@Component (service = ru.entaxy.esb.platform.core.management.profile.api.ConnectorManager.class
		, immediate = true)
public class ConnectorManagerAdapter implements ru.entaxy.esb.platform.core.management.profile.api.ConnectorManager {

	@Reference(service = ConnectorManager.class, cardinality = ReferenceCardinality.MANDATORY
			, collectionType = CollectionType.SERVICE)
	ConnectorManager connectorManager;

	@Reference (service = SystemService.class, cardinality = ReferenceCardinality.MANDATORY
			, collectionType = CollectionType.SERVICE)
	SystemService systemService;	
	
	@Reference (service = ConnectorService.class, cardinality = ReferenceCardinality.MANDATORY
			, collectionType = CollectionType.SERVICE)
	ConnectorService connectorService;
	
	@Override
	public void addConnector(String idOrName, String connectorFactoryId, Map<String, String> paramaters)
			throws Exception {
		
		System system = getSystemByIdOrName(idOrName);
		if (system == null)
			throw new IllegalArgumentException("System not found by Id or Name [" + idOrName + "]");
		ConnectorDto connectorDto = new ConnectorDto();
		connectorDto.setConnectorParams(paramaters);
		connectorDto.setSystemUuid(system.getUuid());
		connectorDto.setTemplateName(connectorFactoryId);
		
		// FIXME remove hardcoded 
		connectorDto.setCreatedBy("admin");
		
		checkBundleResult(connectorManager.installConnector(connectorDto)
				, BundleStatus.ACTIVE);
		
	}

	@Override
	public void removeConnector(String idOrName, String connectorName) throws Exception {

		System system = getSystemByIdOrName(idOrName);
		if (system == null)
			throw new IllegalArgumentException("System not found by Id or Name [" + idOrName + "]");
		
		Connector connector = connectorService.getConnectorByName(connectorName);
		if (connector == null)
			throw new IllegalArgumentException("Connector not found by Name [" + connectorName + "]");
		
		checkBundleResult(connectorManager.uninstallConnector(connector)
				, BundleStatus.UNINSTALL);
	}

	@Override
	public void startConnector(String idOrName, String connectorName) throws Exception {

		System system = getSystemByIdOrName(idOrName);
		if (system == null)
			throw new IllegalArgumentException("System not found by Id or Name [" + idOrName + "]");
		
		Connector connector = connectorService.getConnectorByName(connectorName);
		if (connector == null)
			throw new IllegalArgumentException("Connector not found by Name [" + connectorName + "]");

		checkBundleResult(connectorManager.startConnector(connector)
				, BundleStatus.ACTIVE);
		
	}

	@Override
	public void stopConnector(String idOrName, String connectorName) throws Exception {

		System system = getSystemByIdOrName(idOrName);
		if (system == null)
			throw new IllegalArgumentException("System not found by Id or Name [" + idOrName + "]");
		
		Connector connector = connectorService.getConnectorByName(connectorName);
		if (connector == null)
			throw new IllegalArgumentException("Connector not found by Name [" + connectorName + "]");

		checkBundleResult(connectorManager.stopConnector(connector)
				, BundleStatus.RESOLVED);
		
	}

	protected System getSystemByIdOrName(String idOrName) {
		System result = systemService.getByName(idOrName);
		if (result == null)
			result = systemService.getByUuid(idOrName);
		return result;
	}
	
	protected void checkBundleResult(BundleDto bundleDto, BundleStatus expectedStatus) throws Exception {
		if ((bundleDto == null) || !expectedStatus.equals(bundleDto.getStatus()))
			throw new Exception("Result bundle is null or in a wrong state");
		
	}
}
