/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile;

import ru.entaxy.esb.platform.base.management.core.Qualifier;
import ru.entaxy.esb.platform.base.management.core.api.Attribute;
import ru.entaxy.esb.platform.base.management.core.api.MBeanAnnotated;
import ru.entaxy.esb.platform.base.management.core.api.MBeanExportPolicy;
import ru.entaxy.esb.platform.base.management.core.api.Operation;
import ru.entaxy.esb.platform.base.management.core.utils.BundleAwareMBean;

@MBeanAnnotated(policy = MBeanExportPolicy.ANNOTATION_ENRICH)
public interface ConnectorMBean extends BundleAwareMBean {

	public static interface Helper {

		public static Qualifier getQualifier(String profileName, String connectorName) {
			return ConnectorsMBean.Helper.getQualifier(profileName).qualifier(CONNECTOR_KEY, connectorName);
		}
		
	}

	public static final String CAPABILITY_NAMESPACE = "entaxy.runtime.connector";
	
	public static final String CONNECTOR_KEY = "connector";
	
	public static final String CONNECTOR_KEY_VALUE_DEFAULT = "default";

	// TODO implement if needed
	// public boolean isDefault();

	@Attribute(desc = "Factory ID used to create the connector")
	public String getFactoryId();
	
	@Attribute(desc = "Connector name")
	public String getName();
	
	@Attribute(desc = "Connector registered name used in registry")
	public String getRegisteredName();
	
	@Operation(desc = "Starts the connector")
	public void start() throws Exception;
	
	@Operation(desc = "Stops the connector")
	public void stop() throws Exception;
	
}
