/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile;

import java.util.List;
import java.util.Map;

import javax.management.MBeanOperationInfo;

import ru.entaxy.esb.platform.base.management.core.Qualifier;
import ru.entaxy.esb.platform.base.management.core.api.Attribute;
import ru.entaxy.esb.platform.base.management.core.api.MBeanAnnotated;
import ru.entaxy.esb.platform.base.management.core.api.MBeanExportPolicy;
import ru.entaxy.esb.platform.base.management.core.api.Operation;
import ru.entaxy.esb.platform.base.management.core.api.Parameter;
import ru.entaxy.esb.platform.base.management.core.utils.BundleAwareMBean;

@MBeanAnnotated(policy = MBeanExportPolicy.ANNOTATION_ENRICH)
public interface ProfileMBean extends BundleAwareMBean {

	public static interface Helper {

		public static Qualifier getQualifier(String name) {
			return ProfilesMBean.Q_PROFILES.qualifier(PROFILE_KEY, name);
		}
		
	}

	public static final String CAPABILITY_NAMESPACE = "entaxy.runtime.profile";

	public static final String PROFILE_KEY = "profile";
	
	@Attribute
	public String getName();

	@Attribute
	public String getBasicAuthAccount();
	
	@Attribute
	public String getUuid();
	
	@Attribute(desc = "Systems the current can send messages to")
	public List<String> getAllowedTargets();

	@Attribute(desc = "Systems the current can receive messages from")
	public List<String> getAllowedSources();
	
	
	@Operation(desc = "Adds connector to profile")
	public void addConnector(
			@Parameter(name = "factoryId", desc = "Connector factory ID") String connectorFactoryId
			, @Parameter(name = "parameters", desc = "Connector parameters") Map<String, String> paramaters) throws Exception;
	
	@Operation(desc = "Removes connector from profile")
	public void removeConnector(
			@Parameter(name = "connectorName", desc = "Connector name") String connectorName) throws Exception;
	
	@Operation(desc = "Starts connector in profile")
	public void startConnector(
			@Parameter(name = "connectorName", desc = "Connector name") String connectorName) throws Exception;
	
	@Operation(desc = "Stops connector in profile")
	public void stopConnector(
			@Parameter(name = "connectorName", desc = "Connector name") String connectorName) throws Exception;
	
	@Operation(desc = "Creates account for this system", impact = MBeanOperationInfo.ACTION)
	public void createAccount(
			@Parameter(name = "login", desc = "Login") String login
			, @Parameter(name = "password", desc = "Password") String password) throws Exception;
	
	@Operation(desc = "Allow this system to send messages to other system")
	public void allowTarget(
			@Parameter(name="targetIdOrName", desc = "Target system Id or Name")String idOrName) throws Exception;
	
	@Operation(desc = "Allow this system to receive messages from other system")
	public void allowSource(
			@Parameter(name="sourceIdOrName", desc = "Source system Id or Name")String idOrName) throws Exception;
}
