/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl;

import java.util.Iterator;
import java.util.List;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.utils.ManagedObjectsListener;
import ru.entaxy.esb.platform.core.management.profile.ProfileMBean;
import ru.entaxy.esb.platform.core.management.profile.impl.helper.Helpers;

public class ManagedProfilesListener extends ManagedObjectsListener<List<ManagedProfile>
		, ManagedProfile, ProfileMBean, ProfileMBeanImpl> {

	private static final Logger log = LoggerFactory.getLogger(ManagedProfilesListener.class);
	
	protected Helpers helpers;
	
	public ManagedProfilesListener(BundleContext bundleContext, Helpers helpers) {
		super(bundleContext, ProfileMBean.class);
		this.helpers = helpers;
	}

	@Override
	protected Iterator<ManagedProfile> getIterator(List<ManagedProfile> managedObject) {
		return managedObject.iterator();
	}

	@Override
	protected String getKeyValue(ManagedProfile object) {
		return object.getName();
	}

	@Override
	protected ProfileMBeanImpl createService(ManagedProfile managedObject) {
		try {
			return new ProfileMBeanImpl(bundleContext, managedObject, helpers);
		} catch (Exception e) {
			log.error("Error creating service", e);
			return null;
		}	}

	@Override
	protected String getJmxObjectName(ManagedProfile managedObject) {
		return ProfileMBean.Helper.getQualifier(managedObject.getName()).getValue();
	}

	
	
}
