/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl;

import javax.naming.OperationNotSupportedException;

import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.core.management.profile.api.ProfileManager;

@Component(service = {ProfileHelper.class},
		immediate = true)
public class ProfileHelperImpl implements ProfileHelper {

	private static final Logger log = LoggerFactory.getLogger(ProfileHelperImpl.class);
	
	protected static final String EXPLANATION = "ProfileManager not bound";
	
	protected ProfileManager profileManager = null;
	
	protected ProfileManager nullProfileManager = null;
	
	@Activate
	public void activate(ComponentContext componentContext) {
		
		this.nullProfileManager = new ProfileManager() {
			
			@Override
			public void uninstallProfile(String idOrName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void stopProfile(String idOrName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void startProfile(String idOrName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void createProfile(String id, String name, String description) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
		};
		
	}
	
	
	
	public ProfileManager getEffectiveProfileManager() {
		return profileManager!=null?profileManager:nullProfileManager;
	}

	@Reference (collectionType = CollectionType.SERVICE
			, cardinality = ReferenceCardinality.OPTIONAL
			, policyOption = ReferencePolicyOption.GREEDY
			, service = ProfileManager.class
			, unbind = "unsetProfileManager")
	public void setProfileManager(ProfileManager profileManager) {
		this.profileManager = profileManager;
		log.debug(">> ProfileManager is bound");
	}

	public void unsetProfileManager(ProfileManager profileManager) {
		this.profileManager = null;
		log.debug(">> ProfileManager is unbound");
	}



	@Override
	public void createProfile(String id, String name, String description) throws Exception {
		getEffectiveProfileManager().createProfile(id, name, description);
	}

	@Override
	public void startProfile(String idOrName) throws Exception {
		getEffectiveProfileManager().startProfile(idOrName);
	}

	@Override
	public void stopProfile(String idOrName) throws Exception {
		getEffectiveProfileManager().stopProfile(idOrName);
	}

	@Override
	public void uninstallProfile(String idOrName) throws Exception {
		getEffectiveProfileManager().uninstallProfile(idOrName);
	}

}
