/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import javax.management.NotCompliantMBeanException;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.utils.BundleAwareMBeanImpl;
import ru.entaxy.esb.platform.core.management.profile.ProfileMBean;
import ru.entaxy.esb.platform.core.management.profile.impl.helper.Helpers;
import ru.entaxy.esb.system.auth.basic.jpa.api.entity.BasicAuthAccount;
import ru.entaxy.esb.system.jpa.entity.System;
import ru.entaxy.platform.base.support.CommonUtils;

public class ProfileMBeanImpl extends BundleAwareMBeanImpl<ManagedProfile, ProfileMBean> implements ProfileMBean {

	private static final Logger log = LoggerFactory.getLogger(ProfileMBeanImpl.class);
	
	Helpers helpers;
	
	public ProfileMBeanImpl(BundleContext bundleContext, ManagedProfile managedProfile
				, Helpers helpers) 
			throws NotCompliantMBeanException {
		super(ProfileMBean.class, managedProfile);
		this.bundleContext = bundleContext;
		this.helpers = helpers;
	}
	
	@Override
	public String getName() {
		return managedObject.getName();
	}

	@Override
	public String getUuid() {
		System system = helpers.systemService.getByName(getName());
		if (system == null)
			return "N/A";
		return system.getUuid();
	}
	
	
	
	@Override
	public String getBasicAuthAccount() {
		Optional<BasicAuthAccount> bas = this.helpers.basicAuthService.getBySystem(getUuid());
		if (bas.isPresent())
			return bas.get().getLogin();
		return "";
	}
	
	@Override
	public void createAccount(String login, String password) throws Exception {
		if (CommonUtils.isValid(getBasicAuthAccount()))
			throw new IllegalArgumentException("Account already exists");
		// FIXME remove hardcoded
		this.helpers.basicAuthService.saveCommon(login
				, password
				, getUuid()
				, "admin"
				, "admin");
		
	}
	
	@Override
	public void addConnector(String connectorFactoryId, Map<String, String> paramaters) throws Exception {
		this.helpers.connectorHelper.addConnector(getName(), connectorFactoryId, paramaters);
	}

	@Override
	public void removeConnector(String connectorName) throws Exception {
		this.helpers.connectorHelper.removeConnector(getName(), connectorName);
	}

	@Override
	public void startConnector(String connectorName) throws Exception {
		this.helpers.connectorHelper.startConnector(getName(), connectorName);
	}

	@Override
	public void stopConnector(String connectorName) throws Exception {
		this.helpers.connectorHelper.stopConnector(getName(), connectorName);
	}

	@Override
	public List<String> getAllowedTargets() {
		try {
			return helpers.permissionHelper.getObjectPermissions(getUuid());
		} catch (Exception e) {
			log.warn("getAllowedTargets not succeded", e);
		}
		return new ArrayList<>();
	}

	@Override
	public List<String> getAllowedSources() {
		try {
			return helpers.permissionHelper.getSubjectPermissions(getUuid());
		} catch (Exception e) {
			log.warn("getAllowedSources not succeded", e);
		}
		return new ArrayList<>();
	}
	
	@Override
	public void allowSource(String idOrName) throws Exception {
		helpers.permissionHelper.createPremission(idOrName, getUuid());
	}

	@Override
	public void allowTarget(String idOrName) throws Exception {
		helpers.permissionHelper.createPremission(getUuid(), idOrName);
		
	}
	
}
