/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl.connector;

import java.util.Map;

import javax.naming.OperationNotSupportedException;

import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.core.management.profile.api.ConnectorManager;
import ru.entaxy.esb.platform.core.management.profile.impl.ProfileHelperImpl;

@Component(service = ConnectorHelper.class, immediate = true)
public class ConnectorHelperImpl implements ConnectorHelper {

	private static final Logger log = LoggerFactory.getLogger(ProfileHelperImpl.class);
	
	protected static final String EXPLANATION = "ConnectorManager not bound";
	
	protected ConnectorManager connectorManager = null;
	
	protected ConnectorManager nullConnectorManager = null;

	@Activate
	public void activate(ComponentContext componentContext) {
		this.nullConnectorManager = new ConnectorManager() {
			
			@Override
			public void stopConnector(String idOrName, String connectorName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void startConnector(String idOrName, String connectorName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void removeConnector(String idOrName, String connectorName) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
			
			@Override
			public void addConnector(String idOrName, String connectorFactoryId, Map<String, String> paramaters)
					throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);
			}
		};
	}

	public ConnectorManager getEffectiveConnectorManager() {
		return connectorManager!=null?connectorManager:nullConnectorManager;
	}

	@Reference (collectionType = CollectionType.SERVICE
			, cardinality = ReferenceCardinality.OPTIONAL
			, policyOption = ReferencePolicyOption.GREEDY
			, service = ConnectorManager.class
			, unbind = "unsetConnectorManager")
	public void setConnectorManager(ConnectorManager connectorManager) {
		this.connectorManager = connectorManager;
		log.debug(">> ConnectorManager is bound");
	}

	public void unsetConnectorManager(ConnectorManager connectorManager) {
		this.connectorManager = null;
		log.debug(">> ConnectorManager is unbound");
	}
	
	
	@Override
	public void addConnector(String idOrName, String connectorFactoryId, Map<String, String> paramaters)
			throws Exception {
		getEffectiveConnectorManager().addConnector(idOrName, connectorFactoryId, paramaters);
	}

	@Override
	public void removeConnector(String idOrName, String connectorName) throws Exception {
		getEffectiveConnectorManager().removeConnector(idOrName, connectorName);
	}

	@Override
	public void startConnector(String idOrName, String connectorName) throws Exception {
		getEffectiveConnectorManager().startConnector(idOrName, connectorName);
	}

	@Override
	public void stopConnector(String idOrName, String connectorName) throws Exception {
		getEffectiveConnectorManager().stopConnector(idOrName, connectorName);
	}

}
