/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl.connector;

import java.util.Iterator;
import java.util.List;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.utils.ManagedObjectsListener;
import ru.entaxy.esb.platform.core.management.profile.ConnectorMBean;
import ru.entaxy.esb.platform.core.management.profile.impl.helper.Helpers;

public class ManagedConnectorsListener extends ManagedObjectsListener<List<ManagedConnector>, ManagedConnector, ConnectorMBean, ConnectorMBeanImpl> { 

	private static final Logger log = LoggerFactory.getLogger(ManagedConnectorsListener.class);
	
	protected Helpers helpers;
	
	public ManagedConnectorsListener(BundleContext bundleContext, Helpers helpers) {
		super(bundleContext, ConnectorMBean.class);
		this.helpers = helpers;
	}

	@Override
	protected Iterator<ManagedConnector> getIterator(List<ManagedConnector> managedObject) {
		return managedObject.iterator();
	}

	@Override
	protected String getKeyValue(ManagedConnector object) {
		return object.getName();
	}

	@Override
	protected ConnectorMBeanImpl createService(ManagedConnector managedObject) {
		try {
			return new ConnectorMBeanImpl(bundleContext, managedObject, helpers);
		} catch (Exception e) {
			log.error("Error creating service", e);
			return null;
		}
	}

	@Override
	protected String getJmxObjectName(ManagedConnector managedObject) {
		return ConnectorMBean.Helper.getQualifier(
				managedObject.getSystem(), 
				managedObject.getName()
			).getValue();
	}

}
