/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl.permission;

import java.util.List;

import javax.naming.OperationNotSupportedException;

import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.core.management.profile.api.PermissionManager;

@Component(service = PermissionHelper.class, immediate = true)
public class PermissionHelperImpl implements PermissionHelper {

	private static final Logger log = LoggerFactory.getLogger(PermissionHelperImpl.class);
	
	protected static final String EXPLANATION = "PermissionManager not bound";
	
	protected PermissionManager permissionManager = null;
	
	protected PermissionManager nullPermissionManager = null;

	@Activate
	public void activate(ComponentContext componentContext) {
		this.nullPermissionManager = new PermissionManager() {
			
			@Override
			public List<String> getSubjectPermissions(String subjectId) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);			}
			
			@Override
			public List<String> getObjectPermissions(String objectId) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);			}
			
			@Override
			public void createPremission(String objectId, String subjectId) throws Exception {
				throw new OperationNotSupportedException(EXPLANATION);			}
		};
	}
	
	public PermissionManager getEffectivePermissionManager() {
		return this.permissionManager!=null?this.permissionManager:this.nullPermissionManager;
	}
	
	@Reference (collectionType = CollectionType.SERVICE
			, cardinality = ReferenceCardinality.OPTIONAL
			, policyOption = ReferencePolicyOption.GREEDY
			, service = PermissionManager.class
			, unbind = "unsetPermissionManager")
	public void setPermissionManager(PermissionManager permissionManager) {
		this.permissionManager = permissionManager;
		log.debug(">> PermissionManager is bound");
	}

	public void unsetPermissionManager(PermissionManager permissionManager) {
		this.permissionManager = null;
		log.debug(">> PermissionManager is unbound");
	}
	
	
	
	@Override
	public List<String> getObjectPermissions(String objectId) throws Exception {
		return getEffectivePermissionManager().getObjectPermissions(objectId);
	}

	@Override
	public List<String> getSubjectPermissions(String subjectId) throws Exception {
		return getEffectivePermissionManager().getSubjectPermissions(subjectId);
	}

	@Override
	public void createPremission(String objectId, String subjectId) throws Exception {
		getEffectivePermissionManager().createPremission(objectId, subjectId);
	}

}
