/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.impl.route;

import java.util.Iterator;
import java.util.List;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.utils.ManagedObjectsListener;
import ru.entaxy.esb.platform.core.management.profile.DefaultRouteMBean;

public class ManagedRoutesListener extends ManagedObjectsListener<List<ManagedRoute>, ManagedRoute, DefaultRouteMBean, DefaultRouteMBeanImpl> {

	private static final Logger log = LoggerFactory.getLogger(ManagedRoutesListener.class);
	
	public ManagedRoutesListener(BundleContext bundleContext) {
		super(bundleContext, DefaultRouteMBean.class);
	}

	@Override
	protected Iterator<ManagedRoute> getIterator(List<ManagedRoute> managedObject) {
		log.debug(">> getIterator");
		return managedObject.iterator();
	}

	@Override
	protected String getKeyValue(ManagedRoute object) {
		return object.getName();
	}

	@Override
	protected DefaultRouteMBeanImpl createService(ManagedRoute managedObject) {
		try {
			return new DefaultRouteMBeanImpl(bundleContext, managedObject);
		} catch (Exception e) {
			log.error("Error creating service", e);
			return null;
		}
	}

	@Override
	protected String getJmxObjectName(ManagedRoute managedObject) {
		return DefaultRouteMBean.Helper.getQualifier(managedObject.getName()).getValue();
	}
	
}
