[#ftl attributes={"generated.type":"blueprint"}]
[#--

    ~~~~~~licensing~~~~~~
    connector
    ==========
    Copyright (C) 2020 - 2021 EmDev LLC
    ==========
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
    ~~~~~~/licensing~~~~~~

--]
<?xml version="1.0" encoding="UTF-8"?>
<blueprint xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
           xmlns:cm="http://aries.apache.org/blueprint/xmlns/blueprint-cm/v1.1.0"
           xmlns:soap-type="http://www.entaxy.ru/ExchangeTypes/1.0"
           xsi:schemaLocation="http://www.osgi.org/xmlns/blueprint/v1.0.0 https://www.osgi.org/xmlns/blueprint/v1.0.0/blueprint.xsd">

    <cm:property-placeholder persistent-id="uniform.service.support"
                             update-strategy="reload">
        <cm:default-properties>
            <cm:property name="redelivery.maximumRedeliveries" value="2"/>
            <cm:property name="redelivery.deadLetterQueue" value="revert.message.dead"/>
            <cm:property name="acknowledge.completion.timeout" value="600000"/>

            <cm:property name="bus.id" value="FFFFFFFF-FFFF-FFFF-FFFF-FFFFFFFFFFFF"/>
            <cm:property name="get.destination" value=""/>
            <cm:property name="test.empty.rate" value="20"/>
            <cm:property name="acknowledge.aggregation.repository" value="jdbcAggregationRepository"/>
            <cm:property name="error.system.name" value="error"/>
        </cm:default-properties>
    </cm:property-placeholder>

	<reference id="entaxy-broker" interface="org.apache.camel.Component"
				filter="(connection.name=entaxy-broker)"/>

    <bean id="inConnector" class="ru.entaxy.esb.system.profile.commons.connectors.in.DirectVMInConnectorImpl">
        <property name="endpointName" value="uniform-service"/>
        <property name="systemName" value="[=systemName]"/>
        <property name="params">
            <map>
                <entry key="isActive" value="[=isActive]"/>
                <entry key="period" value="[=period]"/>
                <entry key="username" value="[=username]"/>
                <entry key="password" value="[=password]"/>
                <entry key="endpointUrl" value="[=endpointUrl]"/>
                <entry key="connectionTimeout" value="[=connectionTimeout]"/>
                <entry key="connectionMaxThreads" value="[=connectionMaxThreads]"/>
            </map>
        </property>
    </bean>
    <service ref="inConnector" auto-export="interfaces">
        <service-properties>
            <entry key="endpointName" value="uniform-service"/>
            <entry key="systemName" value="[=systemName]"/>
        </service-properties>
    </service>

    <bean id="forced" class="java.lang.IllegalArgumentException">
        <argument index="0" value="Content is invalid"/>
    </bean>

    <bean id="uuidGenerator" class="java.util.UUID" scope="prototype" factory-method="randomUUID"/>

    <bean id="headersConverter" class="ru.entaxy.esb.system.common.util.HeadersConverter">
        <property name="namespace" value="http://www.entaxy.ru/ExchangeTypes/1.0"/>
        <property name="customHeaders" value="NTX_CustomHeaders"/>
        <property name="customHeaderPrefix" value="NTX_UNIFORM_EXCHANGE_CustomHeader"/>
    </bean>
    
    <bean id="headerMergeAggregator" class="ru.entaxy.esb.system.common.aggregation.HeaderMergeAggregatorImpl"/>

    <camelContext id="uniform-service-in-connector-[=systemName]-context" xmlns="http://camel.apache.org/schema/blueprint"
                  errorHandlerRef="commonErrorHandler">

        <errorHandler id="commonErrorHandler" redeliveryPolicyRef="noRedelivery"
                      type="DeadLetterChannel" deadLetterUri="direct-vm:commonErrorEndpoint"/>
        <redeliveryPolicyProfile id="noRedelivery" disableRedelivery="true"/>

        [#if isActive == "true"]
            <route id="uniform-service-active-in-connector">
                <from uri="timer://get-package?synchronous=true&amp;period=[=period]"/>
                <setHeader name="ENTAXY_SystemId">
                    <constant>[=systemName]</constant>
                </setHeader>
                <setHeader name="ENTAXY_SystemEndpoint">
                    <constant>[=endpointUrl]</constant>
                </setHeader>
                <to uri="direct:uniform-service-connector-get-package-active?block=true&amp;timeout=60000"/>
            </route>

            <!--In-->
            <route id="get-package-active" streamCache="true" >
                <from uri="direct:uniform-service-connector-get-package-active"/>
                <when>
                    <simple>${headers.ENTAXY_SystemEndpoint} == null || ${headers.ENTAXY_SystemId} == null</simple>
                    <log message="ENTAXY_SystemEndpoint is not set" loggingLevel="ERROR"/>
                    <stop/>
                </when>
                <setHeader name="SystemHasMessages">
                    <constant>true</constant>
                </setHeader>

                <setHeader name="operationNamespace">
                    <constant>http://www.entaxy.ru/connector/universal/system/1.0</constant>
                </setHeader>
                <setHeader name="CamelDestinationOverrideUrl">
                    <simple>${headers.ENTAXY_SystemEndpoint}</simple>
                </setHeader>

                <loop doWhile="true" copy="true">
                    <to uri="direct:get-package"/>
                </loop>
            </route>

            <route id="direct:get-package">
                <from uri="direct:get-package"/>
                <simple resultType="Boolean">${headers.SystemHasMessages} == 'true'</simple>

                <setHeader name="NTX_BusUUID">
                    <constant>{{bus.id}}</constant>
                </setHeader>
                <setHeader name="ENTAXY_Destination">
                    <constant>{{get.destination}}</constant>
                </setHeader>
                <to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/GetPacketRequest.xsl?failOnNullBody=false"/>
                <removeHeader headerName="NTX_BusUUID"/>
                <removeHeader headerName="ENTAXY_Destination"/>

                <setHeader name="operationName">
                    <constant>GetPackets</constant>
                </setHeader>
                <to uri="direct-vm:uniform-service-send-package-active"/>
                <removeHeader headerName="SOAPAction"/>

                <setBody>
                    <xpath>//soap-type:response/soap-type:packets</xpath>
                </setBody>

                <choice>
                    <when>
                        <xpath>//soap-type:packets/soap-type:packet[1]/soap-type:message/soap-type:content/node()
                        </xpath>
                        <setBody>
                            <xpath>//soap-type:packets/soap-type:packet[1]</xpath>
                        </setBody>

                        <to uri="entaxy-broker:queue:entaxy.uniform.service.exchange.send.to.profile?exchangePattern=InOnly"/>

                        <setHeader name="NTX_UUID">
                            <simple>${headers.ENTAXY_MessageUUID}</simple>
                        </setHeader>
                        <to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/ConfirmGettingPacketRequest.xsl?failOnNullBody=false"/>
                        <setHeader name="operationName">
                            <constant>ConfirmGettingPackets</constant>
                        </setHeader>

                        <to uri="direct-vm:uniform-service-send-package-active"/>
                        <removeHeader headerName="SOAPAction"/>

                        <when>
                            <xpath>not(//soap-type:status/text() = '200')</xpath>
                            <log message="ConfirmGettingPacket fail" loggingLevel="ERROR"/>
                            <!--                            TODO что делать с сообщением в очереди ? -->
                        </when>
                        <setHeader name="SystemHasMessages">
                            <constant>true</constant>
                        </setHeader>
                    </when>
                    <otherwise>
                        <setHeader name="SystemHasMessages">
                            <constant>false</constant>
                        </setHeader>
                        <log message="System ${headers.NTX_SystemId} does not have messages" loggingLevel="INFO"/>
                    </otherwise>
                </choice>
            </route>

            <route id="soap-send-to-profile">
                <from uri="entaxy-broker:queue:entaxy.uniform.service.exchange.send.to.profile"/>
                <setHeader name="ENTAXY_MessageUUID">
                    <xpath resultType="String">//soap-type:packet/soap-type:message/soap-type:messageUUID/text()</xpath>
                </setHeader>
                <setHeader name="ENTAXY_Destination">
                    <xpath resultType="String">
                        //soap-type:packet/soap-type:header/soap-type:destination/soap-type:id/text()
                    </xpath>
                </setHeader>
                <setHeader name="ENTAXY_DestinationType">
                    <xpath resultType="String">
                        //soap-type:packet/soap-type:header/soap-type:destination/soap-type:type/text()
                    </xpath>
                </setHeader>
                <setHeader name="ENTAXY_Source">
                    <xpath resultType="String">//soap-type:packet/soap-type:message/soap-type:source/soap-type:id/text()
                    </xpath>
                </setHeader>
                <setHeader name="ENTAXY_SourceType">
                    <xpath resultType="String">//soap-type:packet/soap-type:message/soap-type:source/soap-type:type/text()
                    </xpath>
                </setHeader>

                <choice>
                    <when>
                        <simple>${headers.ENTAXY_Source} == ''</simple>
                        <setHeader name="ENTAXY_Source">
                            <simple>${headers.X-SystemName}</simple>
                        </setHeader>
                    </when>
                    <when>
                        <simple>${headers.NTX_SystemId} != ${headers.ENTAXY_Source}</simple>
                        <log message="Source name header and SystemId is not equals!" loggingLevel="WARN"/>
                    </when>
                </choice>
                <setHeader name="ENTAXY_MessageType">
                    <xpath resultType="String">//soap-type:packet/soap-type:message/soap-type:messageType/text()</xpath>
                </setHeader>
                <setHeader name="ENTAXY_Priority">
                    <xpath resultType="Integer">//soap-type:packet/soap-type:message/soap-type:priority/text()</xpath>
                </setHeader>
                <when>
                    <simple>${headers.ENTAXY_Priority} == null || ${headers.ENTAXY_Priority} == ""</simple>
                    <setHeader name="ENTAXY_Priority">
                        <simple resultType="Integer">4</simple>
                    </setHeader>
                </when>

                <setHeader name="NTX_CustomHeaders">
                    <xpath>//soap-type:packet/soap-type:message/soap-type:customHeader</xpath>
                </setHeader>

                <to uri="direct:get-content-and-content-type"/>
                <when>
                    <simple>${body} == null</simple>
                    <log message="Content is invalid" loggingLevel="ERROR"/>
                    <throwException ref="forced"/>
                </when>
                <removeHeaders pattern="*" excludePattern="NTX_.+|CamelJms.+|ENTAXY_.+"/>
                <to uri="direct-vm:profile-[=systemName]-exit-dispatcher?block=false"/>
            </route>

        [#else]
            <route id="uniform-service-in-connector">
                <from uri="direct-vm:uniform-service-in-connector-[=systemName]"/>
                <log message="Start uniform exchange connector ${headers.operationName}" loggingLevel="DEBUG"/>
                <setHeader name="ENTAXY_ConnectorType">
                    <simple>uniform-service</simple>
                </setHeader>
                <setHeader name="ENTAXY_ConnectorName">
                    <simple>uniform-service-in-connector-[=systemName]</simple>
                </setHeader>

                <choice>
                    <when>
                        <simple>${headers.operationName} == 'PutPackets'</simple>
                        <to uri="direct:send-packets"/>
                    </when>
                    <when>
                        <simple>${headers.operationName} == 'ConfirmGettingPackets'</simple>
                        <to uri="direct:confirm-packets"/>
                    </when>
                    <when>
                        <simple>${headers.operationName} == 'GetPackets'</simple>
                        <to uri="direct:get-packets"/>
                    </when>
                    <otherwise>
                        <log message="uniform-service unknown operation: ${headers.operationName}" loggingLevel="WARN"/>
                    </otherwise>
                </choice>
            </route>
        [/#if]
        <route id="send-packets">
            <from uri="direct:send-packets"/>

            <setHeader name="NTX_UNIFORM_EXCHANGE_PacketCount">
                <xpath resultType="Integer">count(/soap-type:packets/soap-type:packet)</xpath>
            </setHeader>
            <when>
                <simple>${headers.NTX_UNIFORM_EXCHANGE_PacketCount} &gt; 1</simple>
                <throwException exceptionType="java.lang.UnsupportedOperationException"
                                message="Not supported more then 1 packet"/>
            </when>

            <setBody>
                <xpath resultType="org.w3c.dom.Document">/soap-type:packets/soap-type:packet[1]</xpath>
            </setBody>

            <setHeader name="ENTAXY_MessageUUID">
                <xpath resultType="String">/soap-type:packet/soap-type:message/soap-type:messageUUID/text()</xpath>
            </setHeader>
            <choice>
                <when>
                    <simple>${headers.ENTAXY_MessageUUID} == "00000000-0000-0000-0000-000000000000"</simple>
[#--                    health check --]
                    <removeHeaders pattern="ENTAXY_.+|NTX_.+|X-.+"/>
                    <setBody>
                        <simple><![CDATA[<test/>]]></simple>
                    </setBody>
                    <to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/PutPacketResponse.xsl?failOnNullBody=false"/>
                </when>
                <otherwise>
                    <setHeader name="ENTAXY_Destination">
                        <xpath resultType="String">
                            /soap-type:packet/soap-type:header/soap-type:destination/soap-type:id/text()
                        </xpath>
                    </setHeader>
                    <setHeader name="ENTAXY_DestinationType">
                        <xpath resultType="String">
                            /soap-type:packet/soap-type:header/soap-type:destination/soap-type:type/text()
                        </xpath>
                    </setHeader>

                    <log message="Receive message ${headers.ENTAXY_MessageUUID} for ${headers.ENTAXY_Destination}"
                         loggingLevel="INFO"/>
                    <setHeader name="ENTAXY_Source">
                        <xpath resultType="String">
                            /soap-type:packet/soap-type:message/soap-type:source/soap-type:id/text()
                        </xpath>
                    </setHeader>
                    <setHeader name="ENTAXY_SourceType">
                        <xpath resultType="String">
                            /soap-type:packet/soap-type:message/soap-type:source/soap-type:type/text()
                        </xpath>
                    </setHeader>
                    <when>
                        <simple>${headers.X-SystemName} != ${headers.ENTAXY_Source}</simple>
                        <log message="Source name header and auth username is not equals!" loggingLevel="WARN"/>
                        <when>
                            <simple>${headers.ENTAXY_Source} == ''</simple>
                            <setHeader name="ENTAXY_Source">
                                <simple>${headers.X-SystemName}</simple>
                            </setHeader>
                        </when>
                    </when>
                    <setHeader name="ENTAXY_MessageType">
                        <xpath resultType="String">/soap-type:packet/soap-type:message/soap-type:messageType/text()
                        </xpath>
                    </setHeader>
                    <setHeader name="ENTAXY_Priority">
                        <xpath resultType="Integer">/soap-type:packet/soap-type:message/soap-type:priority/text()
                        </xpath>
                    </setHeader>
                    <setHeader name="NTX_CustomHeaders">
                        <xpath>/soap-type:packet/soap-type:message/soap-type:customHeader</xpath>
                    </setHeader>
                    <bean ref="headersConverter" method="xml2Json"/>
                    <log message="Headers converted: ${headers}" loggingLevel="DEBUG"/>

                    <to uri="direct:get-content-and-content-type"/>

                    <when>
                        <simple>${body} == null</simple>
                        <log message="Content is invalid" loggingLevel="ERROR"/>
                        <throwException ref="forced"/>
                    </when>
                    <removeHeaders pattern="*" excludePattern="NTX_.+|CamelJms.+|ENTAXY_.+|X-.+"/>

                    <to uri="direct-vm:profile-[=systemName]-exit-dispatcher"/>
                    <removeHeaders pattern="ENTAXY_.+|NTX_.+|X-.+"/>
                    <setBody>
                        <simple><![CDATA[<test/>]]></simple>
                    </setBody>

                    <to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/PutPacketResponse.xsl?failOnNullBody=false"/>
                </otherwise>
            </choice>
        </route>

        <route id="get-content-and-content-type">
            <from uri="direct:get-content-and-content-type"/>

            <setHeader name="ENTAXY_ContentType">
                <xpath resultType="String">/soap-type:packet/soap-type:message/soap-type:contentType/text()
                </xpath>
            </setHeader>
            <choice>
                <when>
                    <xpath>count(/soap-type:packet/soap-type:message/soap-type:content) = 1</xpath>
                    <setHeader name="CamelJms_IntContentXsiType">
                        <xpath resultType="String">
                            /soap-type:packet/soap-type:message/soap-type:content/@xsi:type
                        </xpath>
                    </setHeader>
                    <setHeader name="NTX_ContentHasNotEmptyText">
                        <xpath resultType="Boolean">
                            boolean(normalize-space(/soap-type:packet/soap-type:message/soap-type:content/text()))
                        </xpath>
                    </setHeader>
                    <setHeader name="NTX_ContentChildNodesCount">
                        <xpath resultType="Integer">
                            count(/soap-type:packet/soap-type:message/soap-type:content/child::*)
                        </xpath>
                    </setHeader>
                    <choice>
                        <when>
                            <simple>${headers.NTX_ContentChildNodesCount} == 1</simple>
                            <setBody>
                                <xpath resultType="org.w3c.dom.Document">
                                    /soap-type:packet/soap-type:message/soap-type:content/*
                                </xpath>
                            </setBody>
                            <setHeader name="CamelJms_IntMessageType">
                                <constant>Document</constant>
                            </setHeader>
                        </when>
                        <when>
                            <simple>${headers.NTX_ContentChildNodesCount} > 1</simple>
                            <setBody>
                                <xpath resultType="org.w3c.dom.NodeList">
                                    /soap-type:packet/soap-type:message/soap-type:content/*
                                </xpath>
                            </setBody>
                            <setHeader name="CamelJms_IntMessageType">
                                <constant>NodeList</constant>
                            </setHeader>
                        </when>
                        <when>
                            <simple>${headers.NTX_ContentChildNodesCount} == 0</simple>
                            <choice>
                                <when>
                                    <simple>${headers.NTX_ContentHasNotEmptyText}</simple>
                                    <setBody>
                                        <xpath resultType="String">
                                            /soap-type:packet/soap-type:message/soap-type:content/text()
                                        </xpath>
                                    </setBody>
                                    <setHeader name="CamelJms_IntMessageType">
                                        <constant>String</constant>
                                    </setHeader>
                                </when>
                                <otherwise>
                                    <setBody>
                                        <constant>
                                            <![CDATA[<emptyContent xmlns="http://www.entaxy.ru/connector/universal/internal"/>]]></constant>
                                    </setBody>
                                </otherwise>
                            </choice>
                        </when>
                        <otherwise>
                            <log message="Content is invalid" loggingLevel="ERROR"/>
                            <throwException ref="forced"/>
                        </otherwise>
                    </choice>
                    <removeHeader headerName="NTX_ContentHasNotEmptyText"/>
                    <removeHeader headerName="NTX_ContentChildNodesCount"/>
                </when>
                <otherwise>
                    <setBody>
                        <constant>
                            <![CDATA[<empty xmlns="http://www.entaxy.ru/connector/universal/internal"/>]]></constant>
                    </setBody>
                </otherwise>
            </choice>
        </route>

        <route id="get-packets">
            <from uri="direct:get-packets"/>
            <setHeader name="NTX_SystemId">
                <xpath>/soap-type:getPacketsRequest/soap-type:destination/soap-type:id/text()</xpath>
            </setHeader>
            <choice>
                <when>
                    <simple>${headers.NTX_SystemId} == ''</simple>
                    <setHeader name="NTX_SystemId">
                        <simple>${headers.X-SystemName}</simple>
                    </setHeader>
                </when>
                <when>
                    <simple>${headers.NTX_SystemId} != ${headers.X-SystemName}</simple>
                    <log message="Source name header and SystemId is not equals!" loggingLevel="WARN"/>
                    <throwException exceptionType="java.security.AccessControlException" message="Unauthorized"/>
                </when>
            </choice>

            <setHeader name="NTX_OutQueue">
                <simple>entaxy.uniform.service.exchange.[=systemName]</simple>
            </setHeader>
            <setHeader name="NTX_LimitCount">
                <xpath resultType="Integer">/soap-type:getPacketsRequest/soap-type:limitCount/text()</xpath>
            </setHeader>
            <setHeader name="NTX_LimitSize">
                <xpath resultType="Integer">/soap-type:getPacketsRequest/soap-type:limitSize/text()</xpath>
            </setHeader>
            <when>
                <simple>${headers.NTX_LimitCount} &gt; 1 || ${headers.NTX_LimitSize} &gt; 0</simple>
                <throwException exceptionType="java.lang.UnsupportedOperationException"
                                message="Not supported limitation mode! Only default values."/>
            </when>
            
            <log message="Fetch From JMS queue: ${headers.NTX_OutQueue}; SystemId=${headers.NTX_SystemId}"
                 loggingLevel="INFO"/>
			
			<setBody>
				<simple>${null}</simple>
			</setBody>
			
            <pollEnrich timeout="0" strategyRef="headerMergeAggregator">
                <simple>entaxy-broker:queue:${headers.NTX_OutQueue}</simple>
            </pollEnrich>

            <choice>
                <when>
                    <simple>${body} == null</simple>
                    <log message="No Messages" loggingLevel="WARN"/>
                    <setHeader name="NTX_Status">
                        <constant>204</constant>
                    </setHeader>
                    <to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/GetPacketResponse.xsl?failOnNullBody=false"/>
                    <removeHeaders pattern="NTX_Status"/>
                </when>
                <otherwise>
                    <log message="Get Message ${body}" loggingLevel="DEBUG"/>

                    <bean ref="headersConverter" method="json2xml"/>
                    <setHeader name="ENTAXY_AcknowledgeMsgID">
                        <simple>${bean:uuidGenerator.toString}</simple>
                    </setHeader>

                    <to uri="direct-vm:uniform-service-common-save-acknowledge?block=true&amp;timeout=70000"/>
                    <when>
                        <simple>${headers.CamelJms_IntMessageType} == 'String'</simple>
                        <bean beanType="org.apache.commons.lang3.StringEscapeUtils"
                              method="escapeXml(${body})"/>
                    </when>
                    <setBody>
                        <simple><![CDATA[<body>${body}</body>]]></simple>
                    </setBody>

                    <to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/GetPacketResponse.xsl"/>
                    <to uri="direct-vm:uniform-service-transform-body?block=true&amp;timeout=30000"/>
                    <log message="Send message ${headers.ENTAXY_MessageUUID} for ${headers.NTX_SystemId} ackId ${headers.ENTAXY_AcknowledgeMsgID}"
                         loggingLevel="INFO"/>
                    <removeHeaders pattern="JMS.+|NTX_.+|ENTAXY_.+"/>
                </otherwise>
            </choice>
        </route>

        <route id="confirm-packets">
            <from uri="direct:confirm-packets"/>

            <setHeader name="NTX_UuidCount">
                <xpath resultType="Integer">count(/soap-type:uuids/soap-type:uuid)</xpath>
            </setHeader>
            <when>
                <simple>${headers.NTX_UuidCount} &gt; 1</simple>
                <throwException exceptionType="java.lang.UnsupportedOperationException"
                                message="Not supported more then 1 uuid"/>
            </when>
            <setBody>
                <xpath resultType="org.w3c.dom.Document">/soap-type:uuids/soap-type:uuid[1]</xpath>
            </setBody>

            <setHeader name="ENTAXY_AcknowledgeMsgID">
                <xpath resultType="String">/soap-type:uuid/text()</xpath>
            </setHeader>
            <to uri="direct-vm:uniform-service-receive-acknowledge"/>
            <setHeader name="NTX_UUID">
                <simple>${headers.ENTAXY_AcknowledgeMsgID}</simple>
            </setHeader>
            <to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/ConfirmGettingPacketResponse.xsl?failOnNullBody=false"/>
            <removeHeaders pattern="Authorization|breadcrumbId|NTX_.+|ENTAXY_.+"/>
        </route>
    </camelContext>

</blueprint>
