/*-
 * ~~~~~~licensing~~~~~~
 * basic-auth-api
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.auth.basic.jpa.api.entity.field;

import java.util.Arrays;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public enum EncryptionAlgorithm {

    PLAIN("PLAIN"),
    MD5("MD5"),
    SHA1("SHA-1"),
    SHA256("SHA-256"),
    SHA384("SHA-384"),
    SHA512("SHA-512");

    private static Map<String, EncryptionAlgorithm> map = null;
    private final String name;

    EncryptionAlgorithm(String name) {
        this.name = name;
    }

    public String getAlgorithmName() {
        return name;
    }

    public boolean equalsName(String checkingName) {
        return checkingName != null && this.name.equals(checkingName.toUpperCase());
    }

    public static EncryptionAlgorithm getByName(String name) {
        return map.get(name.toUpperCase());
    }

    static {
        map = Arrays.stream(EncryptionAlgorithm.values())
                .collect(Collectors.toMap(EncryptionAlgorithm::getAlgorithmName, Function.identity()));
    }
}
