/*-
 * ~~~~~~licensing~~~~~~
 * htpasswd
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.auth.basic.htpasswd.entity;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import ru.entaxy.esb.system.auth.basic.jpa.api.entity.BasicAuthAccount;

import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

public class Htpasswd {

    private static final Log LOG = LogFactory.getLog(Htpasswd.class);
    private String directory;
    private String fileName;
    private String masterSalt = null;
    private final List<HtpasswdEntry> entries = new ArrayList<>();

    public Htpasswd() {
    }

    public void prepare(List<BasicAuthAccount> accounts) throws NoSuchAlgorithmException {
        if (masterSalt == null || masterSalt.isEmpty()) {
            throw new IllegalArgumentException("masterSalt not setted!");
        }
        if (accounts != null && accounts.size() > 0) {
            entries.clear();
            for (BasicAuthAccount account : accounts) {
                entries.add(new HtpasswdEntry(
                        account.getLogin(),
                        account.getPasswordHash(),
                        masterSalt,
                        account.getEncryptionAlgorithm().getAlgorithmName()));
            }
        }
    }

    public void addString(String login, String passwordHash, String encryptionAlgorithm) throws NoSuchAlgorithmException {
        entries.add(new HtpasswdEntry(
                login,
                passwordHash,
                masterSalt,
                encryptionAlgorithm));
    }

    public String getDirectory() {
        return directory;
    }

    public void setDirectory(String directory) {
        this.directory = directory;
    }

    public String getFileName() {
        return fileName;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public String getMasterSalt() {
        return masterSalt;
    }

    public void setMasterSalt(String masterSalt) {
        this.masterSalt = masterSalt;
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        for (HtpasswdEntry entry : entries) {
            builder.append(entry.toString());
        }
        return builder.toString();
    }

}
