/*-
 * ~~~~~~licensing~~~~~~
 * system-commons
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.common.interceptor;

import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.binding.soap.interceptor.AbstractSoapInterceptor;
import org.apache.cxf.headers.Header;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.jaxb.JAXBDataBinding;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.Phase;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import java.util.List;
import java.util.Map;

public class SoapHeaderInterceptor extends AbstractSoapInterceptor {

    private static final Logger log = LoggerFactory.getLogger(SoapHeaderInterceptor.class);

    private static final String HEADER_USER_LOGIN = "X-ForwardedUser";
    private static final String HEADER_CREATED_BY = "createdBy";
    private String namespaceUri = "";

    public SoapHeaderInterceptor() {
        super(Phase.READ);
    }

    public void handleMessage(SoapMessage message) throws Fault {
        try {
            Map<String, List<String>> headers = (Map<String, List<String>>) message.get(Message.PROTOCOL_HEADERS);
            List<Header> soapHeaders = message.getHeaders();
            soapHeaders.add(new Header(new QName(namespaceUri, HEADER_CREATED_BY),
                    getSystemName(headers), new JAXBDataBinding(String.class)));
            message.put(Header.HEADER_LIST, soapHeaders);
        } catch (JAXBException e) {
            log.error("Error", e);
            throw new Fault(e);
        }
    }

    private String getSystemName(Map<String, List<String>> headers) {
        List<String> list = headers.get(HEADER_USER_LOGIN);
        return list != null && !list.isEmpty() ? headers.get(HEADER_USER_LOGIN).get(0) : null;
    }

    public void setNamespaceUri(String namespaceUri) {
        this.namespaceUri = namespaceUri;
    }
}
