/*-
 * ~~~~~~licensing~~~~~~
 * system-commons
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.common.osgi.impl;

import org.osgi.framework.ServiceReference;
import ru.entaxy.esb.system.common.osgi.ReferenceListener;

import java.util.ArrayList;
import java.util.List;

public class CommonReferenceListener<T> implements ReferenceListener<T> {

    protected List<T> registeredReferences = new ArrayList<T>();

    public void register(T service) {
        if (service instanceof ServiceReference)
            return;
        registeredReferences.add(service);
        doAfterRegister(service);
    }

    protected void doAfterRegister(T service) {
        // to override
    }

    protected void doBeforeUnregister(T service) {
        // to override
    }

    public void unregister(T service) {
        if (service instanceof ServiceReference)
            return;
        if (registeredReferences.contains(service)) {
            doBeforeUnregister(service);
            registeredReferences.remove(service);
        }
    }

}
