/*-
 * ~~~~~~licensing~~~~~~
 * events-api
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.core.events.jpa.entity;

import ru.entaxy.esb.system.core.events.common.SubscriptionType;
import ru.entaxy.esb.system.jpa.entity.System;

import javax.persistence.*;
import java.util.Objects;

@Entity
@Table(name = "system_subscription_registry")
public class SystemSubscription {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private long id;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "event_topic_id", referencedColumnName = "id")
    private EventTopic eventTopic;

    @Column(name = "type")
    @Enumerated(EnumType.STRING)
    private SubscriptionType type;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "id_system", referencedColumnName = "id")
    private System system;

    @Column(name = "deleted")
    private boolean deleted;

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public EventTopic getEventTopic() {
        return eventTopic;
    }

    public void setEventTopic(EventTopic eventTopic) {
        this.eventTopic = eventTopic;
    }

    public SubscriptionType getType() {
        return type;
    }

    public void setType(SubscriptionType type) {
        this.type = type;
    }

    public System getSystem() {
        return system;
    }

    public void setSystem(System system) {
        this.system = system;
    }

    public boolean isDeleted() {
        return deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    @Override
    public int hashCode() {
        return Objects.hash(eventTopic, id, system, type);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        SystemSubscription other = (SystemSubscription) obj;
        return Objects.equals(eventTopic, other.eventTopic) && id == other.id
                && Objects.equals(system, other.system) && Objects.equals(type, other.type);
    }

    @Override
    public String toString() {
        return "SystemSubscriptionRegistry [id=" + id + ", eventTopic=" + eventTopic + ", type=" + type
                + ", system=" + system + "]";
    }

}
