/*-
 * ~~~~~~licensing~~~~~~
 * events-rest
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.core.events.rest.aggregation;

import org.apache.camel.AggregationStrategy;
import org.apache.camel.Exchange;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ResponseAggregator implements AggregationStrategy {

    private static final Logger LOG = LoggerFactory.getLogger(ResponseAggregator.class);

    @Override
    public Exchange aggregate(Exchange oldExchange, Exchange newExchange) {
        LOG.debug("oldExchange " + (oldExchange != null));
        LOG.debug("newExchange " + (newExchange != null));

        if (oldExchange == null) {
            oldExchange = newExchange;
            oldExchange.getIn().setBody("[" + oldExchange.getIn().getBody(String.class));
        } else if (newExchange != null) {
            String oldBody = oldExchange.getIn().getBody(String.class);
            String newBody = newExchange.getIn().getBody(String.class);
            oldExchange.getIn().setBody(oldBody + "," + newBody);
        }
        if (newExchange.getProperty("CamelSplitComplete", Boolean.class)) {
            oldExchange.getIn().setBody(oldExchange.getIn().getBody(String.class) + "]");
        }
        return oldExchange;
    }

}
