/*-
 * ~~~~~~licensing~~~~~~
 * permission-component
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.core.permission.component.operation;


import org.apache.camel.Exchange;
import org.osgi.framework.FrameworkUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.system.common.osgi.OSGIUtils;
import ru.entaxy.esb.system.core.permission.component.PermissionEndpoint;
import ru.entaxy.esb.system.core.permission.component.PermissionProducer;
import ru.entaxy.esb.system.core.permission.jpa.PermissionService;
import ru.entaxy.esb.system.jpa.SystemService;

public abstract class BaseOperation implements Operation {

    private static final Logger LOG = LoggerFactory.getLogger(BaseOperation.class);

    private PermissionService permissionService;

    private SystemService systemService;

    @Override
    public void process(Exchange exchange, PermissionEndpoint endpoint) throws Exception {
        doProcess(exchange, endpoint);
    }

    protected abstract void doProcess(Exchange exchange, PermissionEndpoint endpoint) throws Exception;

    protected void validateParams(String... parameters) {
        for (String parameter : parameters) {
            if (parameter == null || "0".equals(parameter) || parameter.isEmpty()) {
                throw new IllegalArgumentException("One or more parameters empty!");
            }
        }
    }

    protected PermissionService getPermissionService() {
        if (permissionService == null) {
            permissionService = (PermissionService) OSGIUtils.getServiceReference(
                    FrameworkUtil.getBundle(PermissionProducer.class).getBundleContext(),
                    PermissionService.class.getName());
        }
        return permissionService;
    }

    protected SystemService getSystemService() {
        if (systemService == null) {
            systemService = (SystemService) OSGIUtils.getServiceReference(
                    FrameworkUtil.getBundle(PermissionProducer.class).getBundleContext(),
                    SystemService.class.getName());
        }
        return systemService;
    }

}
