/*-
 * ~~~~~~licensing~~~~~~
 * profile-commons
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.profile.commons;

import org.apache.camel.Exchange;
import org.apache.camel.Header;
import ru.entaxy.esb.system.common.exception.ConnectorNotFound;
import ru.entaxy.esb.system.common.osgi.impl.CommonNamedReferenceListener;
import ru.entaxy.esb.system.profile.commons.connectors.Connector;

public class ConnectorRegistry extends CommonNamedReferenceListener<Connector> {

    public String getParam(String systemName, String paramName) {
        Connector connector = getReference(systemName);
        if (connector != null) {
            return connector.getParam(paramName);
        } else {
            throw new ConnectorNotFound("Connector for " + systemName + " not found");
        }
    }

    public void send(@Header("X-SystemName") String systemName, Exchange exchange) {
        Connector connector = getReference(systemName);
        if (connector != null) {
            connector.send(exchange);
        } else {
            throw new ConnectorNotFound("Connector for " + systemName + " not found");
        }
    }

    @Override
    protected String getObjectName(Connector Connector) {
        return Connector.getSystemName();
    }
}

