/*-
 * ~~~~~~licensing~~~~~~
 * system-group-component
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.groups.component;

import org.apache.camel.Exchange;
import org.apache.camel.support.DefaultProducer;
import org.osgi.framework.BundleContext;
import org.osgi.framework.FrameworkUtil;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.system.common.exception.ProfileNotFound;
import ru.entaxy.esb.system.groups.registry.system.groups.profile.SystemGroupCollectorListener;
import ru.entaxy.esb.system.groups.registry.system.groups.profile.SystemGroupProfile;


public class SystemGroupProducer extends DefaultProducer {

    private static final Logger LOG = LoggerFactory.getLogger(SystemGroupProducer.class);
    private final SystemGroupEndpoint endpoint;

    private SystemGroupCollectorListener systemGroupCollectorListener;

    public SystemGroupProducer(SystemGroupEndpoint endpoint) {
        super(endpoint);
        this.endpoint = endpoint;
    }

    public void process(Exchange exchange) throws Exception {
        LOG.debug("In SystemGroupProducer " + exchange.getIn().getBody());

        //get System Group Profile from systemRegistry endpoint.getSystemName()
        SystemGroupProfile systemGroupProfile = null;

        systemGroupProfile = getSystemGroupProfile(endpoint.getSystemName());

        LOG.debug("Called system group profile " + (systemGroupProfile != null ? systemGroupProfile.getProfileName() : "NULL"));

        systemGroupProfile.send(exchange);
    }

    public SystemGroupProfile getSystemGroupProfile(String nameSystem) throws ProfileNotFound {
        systemGroupCollectorListener = getSystemGroupCollectorListener();
        LOG.info("Registry SystemGroupProfile {}; services count {}", nameSystem, systemGroupCollectorListener.getReferenceNames().size());
        SystemGroupProfile systemGroupProfile = null;
        if (systemGroupCollectorListener.isRegistered(nameSystem)) {
            systemGroupProfile = (SystemGroupProfile) systemGroupCollectorListener.getReference(nameSystem);
        } else {
            throw new ProfileNotFound("Profile for group system " + nameSystem + " not found");
        }
        return systemGroupProfile;
    }


    public SystemGroupCollectorListener getSystemGroupCollectorListener() {
        if (systemGroupCollectorListener == null) {
            BundleContext bundleContext = FrameworkUtil.getBundle(SystemGroupProducer.class).getBundleContext();
            ServiceReference<SystemGroupCollectorListener> systemGroupCollectorListenerServiceReference =
                    bundleContext.getServiceReference(SystemGroupCollectorListener.class);
            systemGroupCollectorListener = bundleContext.getService(systemGroupCollectorListenerServiceReference);
        }
        return systemGroupCollectorListener;
    }

}
