/*-
 * ~~~~~~licensing~~~~~~
 * system-group-impl
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.groups.registry.jpa.impl;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import ru.entaxy.esb.system.groups.registry.jpa.SystemGroupRegistryService;
import ru.entaxy.esb.system.groups.registry.jpa.entity.SystemGroupRegistry;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import javax.transaction.Transactional;
import java.util.Date;
import java.util.List;

/**
 * Implementation of the SystemGroupRegistry service using the JPA entity manager service (provided by Karaf).
 */
@Transactional
public class SystemGroupRegistryServiceImpl implements SystemGroupRegistryService {

    private SessionFactory sessionFactory;

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public Session getSession() {
        return sessionFactory.openSession();
    }

    @Override
    public List<SystemGroupRegistry> list() {
        List<SystemGroupRegistry> list;
        try (Session s = getSession()) {
            s.getTransaction().begin();
            CriteriaQuery<SystemGroupRegistry> cq = s.getCriteriaBuilder().createQuery(SystemGroupRegistry.class);
            cq.from(SystemGroupRegistry.class);
            list = s.createQuery(cq).getResultList();
            s.getTransaction().commit();
            s.close();
        }

        return list;
    }

    @Override
    public SystemGroupRegistry get(long id) {
        SystemGroupRegistry systemGroupRegistry;
        try (Session s = getSession()) {
            s.getTransaction().begin();

            CriteriaBuilder builder = s.getCriteriaBuilder();
            CriteriaQuery<SystemGroupRegistry> criteriaQuery = builder.createQuery(SystemGroupRegistry.class);
            Root<SystemGroupRegistry> root = criteriaQuery.from(SystemGroupRegistry.class);
            criteriaQuery.where(builder.equal(root.get("id"), id));
            systemGroupRegistry = s.createQuery(criteriaQuery).getSingleResult();

            s.getTransaction().commit();
            s.close();
        }
        return systemGroupRegistry;
    }

    @Override
    public SystemGroupRegistry getByUuid(String uuid) {
        SystemGroupRegistry systemGroupRegistry;
        try (Session s = getSession()) {
            s.getTransaction().begin();

            CriteriaBuilder builder = s.getCriteriaBuilder();
            CriteriaQuery<SystemGroupRegistry> criteriaQuery = builder.createQuery(SystemGroupRegistry.class);
            Root<SystemGroupRegistry> root = criteriaQuery.from(SystemGroupRegistry.class);
            criteriaQuery.where(builder.equal(root.get("uuid"), uuid));
            systemGroupRegistry = s.createQuery(criteriaQuery).getSingleResult();

            s.getTransaction().commit();
            s.close();
        }
        return systemGroupRegistry;
    }

    @Override
    public SystemGroupRegistry add(SystemGroupRegistry systemGroupRegistry) {
        try (Session s = getSession()) {
            s.getTransaction().begin();
            s.persist(systemGroupRegistry);
            s.getTransaction().commit();
            s.close();
        }
        return systemGroupRegistry;
    }

    @Override
    public SystemGroupRegistry add(String uuid, String name, String description, Date createDate, Date editDate,
                                   String createdBy, String editedBy, double version, String type, String connectorId,
                                   String descriptionFileUrl, String endpointName, String address, String responseType) {
        SystemGroupRegistry systemGroupRegistry = new SystemGroupRegistry();
        systemGroupRegistry.setUuid(uuid);
        systemGroupRegistry.setName(name);
        systemGroupRegistry.setDescription(description);
        systemGroupRegistry.setCreateDate(createDate);
        systemGroupRegistry.setEditDate(editDate);
        systemGroupRegistry.setCreatedBy(createdBy);
        systemGroupRegistry.setVersion(version);
        systemGroupRegistry.setType(type);
        systemGroupRegistry.setConnectorId(connectorId);
        systemGroupRegistry.setDescriptionFileUrl(descriptionFileUrl);
        systemGroupRegistry.setEndpointName(endpointName);
        systemGroupRegistry.setAddress(address);
        systemGroupRegistry.setResponseType(responseType);

        add(systemGroupRegistry);

        return systemGroupRegistry;
    }

    @Override
    public void remove(long id) {
        try (Session s = getSession()) {
            s.getTransaction().begin();

            SystemGroupRegistry systemGroupRegistry = get(id);
            s.delete(SystemGroupRegistry.class.getName(), systemGroupRegistry);

            s.getTransaction().commit();
            s.close();
        }
    }

}
