/*-
 * ~~~~~~licensing~~~~~~
 * system-api
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.jpa.entity;

import com.sun.istack.NotNull;
import ru.entaxy.esb.system.connector.entity.Connector;
import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleEntity;

import javax.persistence.*;
import java.util.Date;
import java.util.Objects;
import java.util.Set;

@NamedEntityGraph(
        name = "system-entity-graph",
        attributeNodes = {
                @NamedAttributeNode(value = "connectors", subgraph = "connectors-subgraph"),
                @NamedAttributeNode("exportAlloweds"),
                @NamedAttributeNode("bundleEntity"),
        },
        subgraphs = {
                @NamedSubgraph(
                        name = "connectors-subgraph",
                        attributeNodes = {
                                @NamedAttributeNode("connectorParams"),
                                @NamedAttributeNode("bundleEntity")
                        }
                )
        }
)
@Entity
@Table(name = "system")
public class System {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private long id;
    @NotNull
    @Column(name = "uuid")
    private String uuid;
    @NotNull
    @Column(name = "name")
    private String name;
    @Column(name = "description")
    private String description;
    @NotNull
    @Column(name = "create_date")
    private Date createDate;
    @Column(name = "edit_date")
    private Date editDate;
    @NotNull
    @Column(name = "created_by")
    private String createdBy;
    @Column(name = "edited_by")
    private String editedBy;
    @Column(name = "version")
    private String version;
    @Column(name = "type")
    private String type;
    @Column(name = "origin")
    private String origin;
    @Column(name = "default_route")
    private Boolean isDefaultRoute = true;
    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    @JoinColumn(name = "system_id")
    private Set<Connector> connectors;
    @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.LAZY)
    @JoinColumn(name = "system_id")
    private Set<ExportAllowed> exportAlloweds;
    @OneToOne(cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.LAZY)
    @JoinColumn(name = "bundle_id")
    private BundleEntity bundleEntity;

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Date getCreateDate() {
        return createDate;
    }

    public void setCreateDate(Date createDate) {
        this.createDate = createDate;
    }

    public Date getEditDate() {
        return editDate;
    }

    public void setEditDate(Date editDate) {
        this.editDate = editDate;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }

    public String getEditedBy() {
        return editedBy;
    }

    public void setEditedBy(String editedBy) {
        this.editedBy = editedBy;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getOrigin() {
        return origin;
    }

    public void setOrigin(String origin) {
        this.origin = origin;
    }

    public Set<Connector> getConnectors() {
        return connectors;
    }

    public void setConnectors(Set<Connector> connectors) {
        this.connectors = connectors;
    }

    public Set<ExportAllowed> getExportAlloweds() {
        return exportAlloweds;
    }

    public void setExportAlloweds(Set<ExportAllowed> exportAlloweds) {
        this.exportAlloweds = exportAlloweds;
    }

    public BundleEntity getBundleEntity() {
        return bundleEntity;
    }

    public void setBundleEntity(BundleEntity bundleEntity) {
        this.bundleEntity = bundleEntity;
    }

    public Boolean getDefaultRoute() {
        return isDefaultRoute;
    }

    public void setDefaultRoute(Boolean defaultRoute) {
        isDefaultRoute = defaultRoute;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        System profile = (System) o;
        return id == profile.id &&
                Objects.equals(version, profile.version) &&
                Objects.equals(name, profile.name) &&
                Objects.equals(description, profile.description) &&
                Objects.equals(createDate, profile.createDate) &&
                Objects.equals(editDate, profile.editDate) &&
                Objects.equals(createdBy, profile.createdBy) &&
                Objects.equals(editedBy, profile.editedBy) &&
                Objects.equals(type, profile.type) &&
                Objects.equals(origin, profile.origin) &&
                Objects.equals(connectors, profile.connectors);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, uuid, name, description, createDate, editDate, createdBy,
                editedBy, version, type, origin, connectors, exportAlloweds);
    }

    @Override
    public String toString() {
        return "System{" +
                "id=" + id +
                ", uuid='" + uuid + '\'' +
                ", name='" + name + '\'' +
                ", description='" + description + '\'' +
                ", createDate=" + createDate +
                ", editDate=" + editDate +
                ", createdBy='" + createdBy + '\'' +
                ", editedBy='" + editedBy + '\'' +
                ", version=" + version +
                ", type='" + type + '\'' +
                ", origin='" + origin +
                '}';
    }
}
