/*-
 * ~~~~~~licensing~~~~~~
 * connector-manager
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.management.connector.manager.mapper;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Mappings;
import org.mapstruct.Named;
import ru.entaxy.esb.system.connector.entity.Connector;
import ru.entaxy.esb.system.connector.entity.ConnectorParam;
import ru.entaxy.esb.system.management.bundle.jpa.dto.BundleDto;
import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleEntity;
import ru.entaxy.esb.system.management.connector.manager.dto.ConnectorDto;

import java.text.SimpleDateFormat;
import java.util.*;

@Mapper(componentModel = "spring")
public interface ConnectorMapper {

    @Mappings({
            @Mapping(target = "createDate", expression = "java(new java.util.Date())"),
            @Mapping(target = "version", source = "name", qualifiedByName = "generateVersion"),
            @Mapping(source = "connectorParams", target = "connectorParams", qualifiedByName = "toConnectorParams")
    })
    Connector toConnectorEntity(ConnectorDto connectorDto);

    @Mappings({
            @Mapping(source = "bundleEntity.status", target = "status"),
            @Mapping(source = "connectorParams", target = "connectorParams", qualifiedByName = "toConnectorParams")
    })
    ConnectorDto toConnectorDto(Connector connector);

    List<ConnectorDto> toConnectorDtos(List<Connector> connectorList);

    @Mappings({
            @Mapping(source = "bundleName", target = "name"),
            @Mapping(source = "bundleUrl", target = "url"),
    })
    BundleEntity toBundleEntity(BundleDto bundle);
    
    @Mappings({
        @Mapping(target = "bundleName", source = "name"),
        @Mapping(target = "bundleUrl", source = "url"),
    })
    BundleDto toBundleDto(BundleEntity bundle);

    default List<ConnectorParam> toConnectorParams(Map<String, String> connectorParams) {
        List<ConnectorParam> connectorParamList = new ArrayList<>();
        for (Map.Entry<String, String> entry : connectorParams.entrySet()) {
            ConnectorParam connectorParam = new ConnectorParam();
            connectorParam.setName(entry.getKey());
            connectorParam.setValue(entry.getValue());
            connectorParamList.add(connectorParam);
        }
        return connectorParamList;
    }

    default Map<String, String> toConnectorParams(List<ConnectorParam> connectorParams) {
        Map<String, String> connectorParamMap = new HashMap<>();
        for (ConnectorParam connectorParam : connectorParams) {
            connectorParamMap.put(connectorParam.getName(), connectorParam.getValue());
        }
        return connectorParamMap;
    }

    @Named("generateVersion")
    default String generateVersion(String version) {
        SimpleDateFormat format = new SimpleDateFormat("1.yyMMdd.HHmmss");
        return format.format(new Date());
    }
}
