/*-
 * ~~~~~~licensing~~~~~~
 * profile-manager
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.system.management.profile.manager.mapper;

import org.mapstruct.InjectionStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Mappings;
import ru.entaxy.esb.system.jpa.entity.ExportAllowed;
import ru.entaxy.esb.system.jpa.entity.System;
import ru.entaxy.esb.system.management.bridge.profile.manager.dto.BridgeProfileDto;
import ru.entaxy.esb.system.management.bundle.jpa.dto.BundleDto;
import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleEntity;
import ru.entaxy.esb.system.management.connector.manager.mapper.ConnectorMapper;
import ru.entaxy.esb.system.management.profile.manager.dto.ProfileDto;

import java.util.List;
import java.util.Set;

@Mapper(componentModel = "spring", uses = {ConnectorMapper.class},
        injectionStrategy = InjectionStrategy.CONSTRUCTOR)
public interface ProfileMapper {

    @Mappings({
            @Mapping(target = "version", source = "version", qualifiedByName = "generateVersion"),
            @Mapping(target = "createDate", expression = "java(new java.util.Date())"),
    })
    System toSystemEntity(ProfileDto profileDto);

    @Mappings({
            @Mapping(target = "status", source = "bundleEntity.status"),
    })
    ProfileDto toProfileDto(System system);

    @Mappings({
            @Mapping(target = "name", source = "bundleName"),
            @Mapping(target = "url", source = "bundleUrl"),
    })
    BundleEntity toBundleEntity(BundleDto bundle);
    
    @Mappings({
        @Mapping(target = "bundleName", source = "name"),
        @Mapping(target = "bundleUrl", source = "url"),
    })
    BundleDto toBundleDto(BundleEntity bundle);

    Set<ExportAllowed> toExportAllowedEntities(Set<String> exportAlloweds);

    Set<String> toExportAlloweds(Set<ExportAllowed> exportAlloweds);

    default ExportAllowed map(String exportAllowed) {
        ExportAllowed exportAllowedEntity = new ExportAllowed();
        exportAllowedEntity.setName(exportAllowed);
        return exportAllowedEntity;
    }

    default String map(ExportAllowed exportAllowed) {
        return exportAllowed != null ? exportAllowed.getName() : null;
    }

    List<ProfileDto> toListProfiles(List<System> systemList);

    @Mappings({
            @Mapping(target = "profileUuid", source = "uuid")
    })
    BridgeProfileDto toBridgeProfileDto(System system);
}
