/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2021 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy.log.debug("loading navigation");
    var TAB_CONFIG = {
        attributes: {
            title: 'Attributes',
            route: '/entaxy/attributes'
        },
        operations: {
            title: 'Operations',
            route: '/entaxy/operations'
        },
        chart: {
            title: 'Chart',
            route: '/entaxy/charts'
        }
    };
    Entaxy._module
    .config(configureRoutes)
    .component('entaxyNavigation', {
        template: `<hawtio-tabs tabs="$ctrl.tabs" on-change="$ctrl.goto(tab)"></hawtio-tabs>`,
        controller: EntaxyNavigationController
    })
    .name;
    Entaxy.log.debug("loaded navigation " + Entaxy.navigationModule);

    function EntaxyNavigationController($scope, $location, workspace, localStorage, jolokia) {
        'ngInject';
        var ctrl = this;
        this.$location = $location;
        entaxyJmxDomain = localStorage['entaxyJmxDomain'] || "ru.entaxy.esb";

        $scope.$on('jmxTreeClicked', function () {
            ctrl.tabs = getTabs();
            var tab = _.find(ctrl.tabs, { path: ctrl.$location.path() });
            if (!tab) {
                tab = ctrl.tabs[0];
            }
            ctrl.$location.path(tab.path);
        });

        EntaxyNavigationController.prototype.$onInit = function () {
            this.tabs = getTabs();
        };

        EntaxyNavigationController.prototype.goto = function (tab) {
            this.$location.path(tab.path);
        };

        function getTabs() {

            var tabs = [];
            var enabledRoutes = Object.keys(TAB_CONFIG)
                .map(function (config) { return TAB_CONFIG[config].route; })
                .filter(function (route) { return _.startsWith(route, '/entaxy'); });
            if (enabledRoutes.length > 0) {
                tabs.push(new Nav.HawtioTab(TAB_CONFIG.attributes.title, TAB_CONFIG.attributes.route));
                tabs.push(new Nav.HawtioTab(TAB_CONFIG.operations.title, TAB_CONFIG.operations.route));
                tabs.push(new Nav.HawtioTab(TAB_CONFIG.chart.title, TAB_CONFIG.chart.route));
            }
            return tabs;
        }

    }
    EntaxyNavigationController.$inject = ['$scope', '$location', 'workspace', 'localStorage', 'jolokia']

    function configureRoutes($routeProvider) {
       $routeProvider.
        when('/entaxy/attributes', { templateUrl: 'plugins/jmx/html/attributes/attributes.html' }).
        when('/entaxy/operations', { template: '<operations></operations>' }).
        when('/entaxy/charts', { templateUrl: 'plugins/jmx/html/charts.html' });
    }
    configureRoutes.$inject = ['$routeProvider'];

})(Entaxy || (Entaxy = {}));
