/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.model;

import org.apache.camel.spi.Metadata;
import org.apache.camel.util.ObjectHelper;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;

/**
 * Allows to declare saga actions to complete or compensate a saga
 */
@Metadata(label = "eip,routing")
@XmlAccessorType(XmlAccessType.FIELD)
public class SagaActionUriDefinition {

    @XmlAttribute(required = true)
    private String uri;

    public SagaActionUriDefinition() {
    }

    public SagaActionUriDefinition(String uri) {
        this.uri = ObjectHelper.notNull(uri, "uri");
    }

    public String getUri() {
        return uri;
    }

    public void setUri(String uri) {
        this.uri = ObjectHelper.notNull(uri, "uri");
    }

    @Override
    public String toString() {
        return uri;
    }

}
