/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.reifier.dataformat;

import org.apache.camel.CamelContext;
import org.apache.camel.model.DataFormatDefinition;
import org.apache.camel.model.dataformat.XStreamDataFormat;

import java.util.Map;

public class XStreamDataFormatReifier extends DataFormatReifier<XStreamDataFormat> {

    public XStreamDataFormatReifier(CamelContext camelContext, DataFormatDefinition definition) {
        super(camelContext, (XStreamDataFormat)definition);
    }

    @Override
    protected void prepareDataFormatConfig(Map<String, Object> properties) {
        properties.put("xstreamDriver", asRef(definition.getDriverRef()));
        properties.put("permissions", definition.getPermissions());
        properties.put("encoding", definition.getEncoding());
        properties.put("converters", definition.getConvertersAsMap());
        properties.put("aliases", definition.getAliasesAsMap());
        properties.put("omitFields", definition.getOmitFieldsAsMap());
        properties.put("implicitCollections", definition.getImplicitCollectionsAsMap());
        properties.put("mode", definition.getMode());
    }

}
